from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSSSLVServer(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSSSLVServer, self).__init__()
        self.options = {'vservername': '',
                        'cipherdetails': '',
                        'cleartextport': '',
                        'dh': '',
                        'dhfile': '',
                        'dhcount': '',
                        'ersa': '',
                        'ersacount': '',
                        'sessreuse': '',
                        'sesstimeout': '',
                        'cipherredirect': '',
                        'crlcheck': '',
                        'cipherurl': '',
                        'sslv2redirect': '',
                        'sslv2url': '',
                        'clientauth': '',
                        'clientcert': '',
                        'sslredirect': '',
                        'redirectportrewrite': '',
                        'nonfipsciphers': '',
                        'ssl2': '',
                        'ssl3': '',
                        'tls1': '',
                        'snienable': '',
                        'service': '',
                        'certkeyname': '',
                        'servicename': '',
                        'ocspcheck': '',
                        'pushenctrigger': ''}

        self.resourcetype = NSSSLVServer.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        """
        Binding object showing the lbmonitor that can be bound to service.
        """
        return "sslvserver"

    # Read/write properties
    def set_vservername(self, vservername):
        """
        The name of the SSL virtual server.
        Minimum length = 1
        """
        self.options['vservername'] = vservername

    def get_vservername(self):
        """
        The name of the SSL virtual server.
        Minimum length = 1
        """
        return self.options['vservername']

    def set_cleartextport(self, cleartextport):
        """
        The port on the back-end web-servers where the clear-text data
        is sent by system. Use this setting for the wildcard IP based
        SSL Acceleration configuration (*:443).
        Minimum value = 1
        """
        self.options['cleartextport'] = cleartextport

    def get_cleartextport(self):
        """
        The port on the back-end web-servers where the clear-text data
        is sent by system. Use this setting for the wildcard IP based
        SSL Acceleration configuration (*:443).
        Minimum value = 1
        """
        return self.options['cleartextport']

    def set_dh(self, dh):
        """
        The state of DH key exchange support for the specified SSL virtual server.
        Default value: DISABLED
        """
        self.options['dh'] = dh

    def get_dh(self):
        """
        The state of DH key exchange support for the specified SSL virtual server.
        Default value: DISABLED
        """
        return self.options['dh']

    def set_dhfile(self, dhfile):
        """
        The file name and path for the DH parameter. The file format is
        PEM. Note: The '-dh' argument must be enabled if this argument
        is specified.
        Minimum length = 1
        """
        self.options['dhfile'] = dhfile

    def get_dhfile(self):
        """
        The file name and path for the DH parameter. The file format is
        PEM. Note: The '-dh' argument must be enabled if this argument
        is specified.
        Minimum length = 1
        """
        return self.options['dhfile']

    def set_dhcount(self, ersa):
        """
        The refresh count for the re-generation of DH public-key and
        private-key from the DH parameter. Zero means infinite usage
        (no refresh). Note: The '-dh' argument must be enabled if this
        argument is specified.
        Default value: 0
        Minimum value = 0
        Maximum value = 65534
        """
        self.options['dhcount'] = ersa

    def get_dhcount(self):
        """
        The refresh count for the re-generation of DH public-key and
        private-key from the DH parameter. Zero means infinite usage
        (no refresh). Note: The '-dh' argument must be enabled if this
        argument is specified.
        Default value: 0
        Minimum value = 0
        Maximum value = 65534
        """
        return self.options['dhcount']

    def set_ersa(self, ersa):
        """
        The state of Ephemeral RSA key exchange support for the SSL
        virtual server.
        Default value: ENABLED
        """
        self.options['ersa'] = ersa

    def get_ersa(self):
        """
        The state of Ephemeral RSA key exchange support for the SSL
        virtual server.
        Default value: ENABLED
        """
        return self.options['ersa']

    def set_ersacount(self, ersacount):
        """
        The refresh count for the re-generation of RSA public-key and
        private-key pair. Zero means infinite usage (no refresh) Note:
        The '-eRSA' argument must be enabled if this argument is
        specified.
        Default value: 0
        Minimum value = 0
        Maximum value = 65534
        """
        self.options['ersacount'] = ersacount

    def get_ersacount(self):
        """
        The refresh count for the re-generation of RSA public-key and
        private-key pair. Zero means infinite usage (no refresh) Note:
        The '-eRSA' argument must be enabled if this argument is
        specified.
        Default value: 0
        Minimum value = 0
        Maximum value = 65534
        """
        return self.options['ersacount']

    def set_sessreuse(self, sessreuse):
        """
        The state of session re-use support for the SSL virtual server.
        Default value: ENABLED
        """
        self.options['sessreuse'] = sessreuse

    def get_sessreuse(self):
        """
        The state of session re-use support for the SSL virtual server.
        Default value: ENABLED
        """
        return self.options['sessreuse']

    def set_sesstimeout(self, sesstimeout):
        """
        The Session timeout value in seconds. The value has to be a
        positive integer. The '-sessReuse' argument must be enabled if
        this argument is specified.
        Default value: 120
        Minimum value = 0
        Maximum value = 0xFFFFFFFE
        """
        self.options['sesstimeout'] = sesstimeout

    def get_sesstimeout(self):
        """
        The Session timeout value in seconds. The value has to be a
        positive integer. The '-sessReuse' argument must be enabled if
        this argument is specified.
        Default value: 120
        Minimum value = 0
        Maximum value = 0xFFFFFFFE
        """
        return self.options['sesstimeout']

    def set_cipherredirect(self, cipherredirect):
        """
        The state of Cipher Redirect feature.
        Default value: DISABLED
        """
        self.options['cipherredirect'] = cipherredirect

    def get_cipherredirect(self):
        """
        The state of Cipher Redirect feature.
        Default value: DISABLED
        """
        return self.options['cipherredirect']

    def set_cipherurl(self, cipherurl):
        """
        The redirect URL to be used with the Cipher Redirect feature.
        """
        self.options['cipherurl'] = cipherurl

    def get_cipherurl(self):
        """
        The redirect URL to be used with the Cipher Redirect feature.
        """
        return self.options['cipherurl']

    def set_sslv2redirect(self, sslv2redirect):
        """
        The state of SSLv2 Redirect feature.
        Default value: DISABLED
        """
        self.options['sslv2redirect'] = sslv2redirect

    def get_sslv2redirect(self):
        """
        The state of SSLv2 Redirect feature.
        Default value: DISABLED
        """
        return self.options['sslv2redirect']

    def set_sslv2url(self, sslv2url):
        """
        The redirect URL to be used with SSLv2 Redirect feature.
        """
        self.options['sslv2url'] = sslv2url

    def get_sslv2url(self):
        """
        The redirect URL to be used with SSLv2 Redirect feature.
        """
        return self.options['sslv2url']

    def set_clientauth(self, clientauth):
        """
        The state of Client-Authentication support for the SSL virtual server.
        Default value: DISABLED
        """
        self.options['clientauth'] = clientauth

    def get_clientauth(self):
        """
        The state of Client-Authentication support for the SSL virtual server.
        Default value: DISABLED
        """
        return self.options['clientauth']

    def set_clientcert(self, clientcert):
        """
        The rule for client authentication. If the clientCert if set to
        Mandatory, the system will terminate the SSL handshake if the
        SSL client does not provide a valid certificate. If the setting
        is Optional, then System will allow SSL clients with no
        certificate or invalid certificates to access the secure
        resource. Note: Make sure proper access control policies are
        defined before changing the above setting to Optional.
        """
        self.options['clientcert'] = clientcert

    def get_clientcert(self):
        """
        The rule for client authentication. If the clientCert if set to
        Mandatory, the system will terminate the SSL handshake if the
        SSL client does not provide a valid certificate. If the setting
        is Optional, then System will allow SSL clients with no
        certificate or invalid certificates to access the secure
        resource. Note: Make sure proper access control policies are
        defined before changing the above setting to Optional.
        """
        return self.options['clientcert']

    def set_sslredirect(self, sslredirect):
        """
        The state of HTTPS redirects for the SSL virtual server. This
        is required for proper working of the redirect messages from
        the web server. The redirect message from the server gives the
        new location for the moved object. This is contained in the
        HTTP header field: Location (for example, Location:
        http://www.moved.org/here.html). For an SSL session, if the
        client browser receives this message, the browser will try to
        connect to the new location. This will break the secure SSL
        session, as the object has moved from a secure site (https://)
        to an unsecured one (http://). Browsers usually flash a warning
        message on the screen and prompt the user to either continue or
        disconnect. When the above feature is enabled, all such http://
        redirect messages are automatically converted to https://. This
        does not break the client SSL session.
        Default value: DISABLED
        """
        self.options['sslredirect'] = sslredirect

    def get_sslredirect(self):
        """
        The state of HTTPS redirects for the SSL virtual server. This
        is required for proper working of the redirect messages from
        the web server. The redirect message from the server gives the
        new location for the moved object. This is contained in the
        HTTP header field: Location (for example, Location:
        http://www.moved.org/here.html). For an SSL session, if the
        client browser receives this message, the browser will try to
        connect to the new location. This will break the secure SSL
        session, as the object has moved from a secure site (https://)
        to an unsecured one (http://). Browsers usually flash a warning
        message on the screen and prompt the user to either continue or
        disconnect. When the above feature is enabled, all such http://
        redirect messages are automatically converted to https://. This
        does not break the client SSL session.
        Default value: DISABLED
        """
        return self.options['sslredirect']

    def set_redirectportrewrite(self, redirectportrewrite):
        """
        The state of port in rewrite while performing HTTPS redirect.
        Default value: DISABLED
        """
        self.options['redirectportrewrite'] = redirectportrewrite

    def get_redirectportrewrite(self):
        """
        The state of port in rewrite while performing HTTPS redirect.
        Default value: DISABLED
        """
        return self.options['redirectportrewrite']

    def set_nonfipsciphers(self, nonfipsciphers):
        """
        The state of usage of non FIPS approved ciphers. Valid only for
        an SSL vserver bound with a FIPS key and certificate.
        Default value: DISABLED
        """
        self.options['nonfipsciphers'] = nonfipsciphers

    def get_nonfipsciphers(self):
        """
        The state of usage of non FIPS approved ciphers. Valid only for
        an SSL vserver bound with a FIPS key and certificate.
        Default value: DISABLED
        """
        return self.options['nonfipsciphers']

    def set_ssl2(self, ssl2):
        """
        The state of SSLv2 protocol support for the SSL virtual server.
        Default value: DISABLED
        """
        self.options['ssl2'] = ssl2

    def get_ssl2(self):
        """
        The state of SSLv2 protocol support for the SSL virtual server.
        Default value: DISABLED
        """
        return self.options['ssl2']

    def set_ssl3(self, ssl3):
        """
        The state of SSLv3 protocol support for the SSL virtual server.
        Default value: ENABLED
        """
        self.options['ssl3'] = ssl3

    def get_ssl3(self):
        """
        The state of SSLv3 protocol support for the SSL virtual server.
        Default value: ENABLED
        """
        return self.options['ssl3']

    def set_tls1(self, tls1):
        """
        The state of TLSv1 protocol support for the SSL virtual server.
        Default value: ENABLED
        """
        self.options['tls1'] = tls1

    def get_tls1(self):
        """
        The state of TLSv1 protocol support for the SSL virtual server.
        Default value: ENABLED
        """
        return self.options['tls1']

    def set_snienable(self, snienable):
        """
        state of SNI feature on virtual server.
        Default value: DISABLED
        """
        self.options['snienable'] = snienable

    def get_snienable(self):
        """
        state of SNI feature on virtual server.
        Default value: DISABLED
        """
        return self.options['snienable']

    def set_pushenctrigger(self, pushenctrigger):
        """
        PUSH packet triggering encryption Always - Any PUSH packet
        triggers encryption Ignore - Ignore PUSH packet for triggering
        encryption Merge - For consecutive sequence of PUSH packets,
        last PUSH packet triggers encryption Timer - PUSH packet
        triggering encryption delayed by timer period defined in 'set
        ssl parameter' .
        """
        self.options['pushenctrigger'] = pushenctrigger

    def get_pushenctrigger(self):
        """
        PUSH packet triggering encryption Always - Any PUSH packet
        triggers encryption Ignore - Ignore PUSH packet for triggering
        encryption Merge - For consecutive sequence of PUSH packets,
        last PUSH packet triggers encryption Timer - PUSH packet
        triggering encryption delayed by timer period defined in 'set
        ssl parameter' .
        """
        return self.options['pushenctrigger']

    def set_cipherdetails(self, cipherdetails):
        """
        Details of the individual ciphers bound to the SSL vserver.
        Select this flag value to display the details of the individual
        ciphers bound to the SSL vserver.
        """
        self.options['cipherdetails'] = cipherdetails

    def get_cipherdetails(self):
        """
        Details of the individual ciphers bound to the SSL vserver.
        Select this flag value to display the details of the individual
        ciphers bound to the SSL vserver.
        """
        return self.options['cipherdetails']

    # Read only properties
    def get_crlcheck(self):
        """
        The state of the CRL check parameter. (Mandatory/Optional)
        """
        return self.options['crlcheck']

    def get_service(self):
        """
        Service
        """
        return self.options['service']

    def get_certkeyname(self):
        """
        The name of the certificate key pair binding.
        """
        return self.options['certkeyname']

    def get_servicename(self):
        """
        Service name.
        """
        return self.options['servicename']

    def get_ocspcheck(self):
        """
        The state of the OCSP check parameter. (Mandatory/Optional)
        """
        return self.options['ocspcheck']

    @staticmethod
    def get(nitro, sslvserver):
        """
        Use this API to fetch sslvserver resource of given name.
        """
        __sslvserver = NSSSLVServer()
        __sslvserver.get_resource(nitro, sslvserver.get_vservername())
        return __sslvserver

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured sslvserver resources.
        """
        __url = nitro.get_url() + NSSSLVServer.get_resourcetype()
        __json_sslvservers = nitro.get(__url).get_response_field(NSSSLVServer.get_resourcetype())
        __sslvservers = []
        for json_sslvserver in __json_sslvservers:
            __sslvservers.append(NSSSLVServer(json_sslvserver))
        return __sslvservers

    @staticmethod
    def update(nitro, sslvserver):
        """
        Use this API to update sslvserver of a given name.
        """
        __sslvserver = NSSSLVServer()
        __sslvserver.set_vservername(sslvserver.get_vservername())
        __sslvserver.set_cleartextport(sslvserver.get_cleartextport())
        __sslvserver.set_dh(sslvserver.get_dh())
        __sslvserver.set_dhfile(sslvserver.get_dhfile())
        __sslvserver.set_dhcount(sslvserver.get_dhcount())
        __sslvserver.set_ersa(sslvserver.get_ersa())
        __sslvserver.set_ersacount(sslvserver.get_ersacount())
        __sslvserver.set_sessreuse(sslvserver.get_sessreuse())
        __sslvserver.set_sesstimeout(sslvserver.get_sesstimeout())
        __sslvserver.set_cipherredirect(sslvserver.get_cipherredirect())
        __sslvserver.set_cipherurl(sslvserver.get_cipherurl())
        __sslvserver.set_sslv2redirect(sslvserver.get_sslv2redirect())
        __sslvserver.set_sslv2url(sslvserver.get_sslv2redirect())
        __sslvserver.set_clientauth(sslvserver.get_clientauth())
        __sslvserver.set_clientcert(sslvserver.get_clientcert())
        __sslvserver.set_sslredirect(sslvserver.get_sslredirect())
        __sslvserver.set_redirectportrewrite(sslvserver.get_redirectportrewrite())
        __sslvserver.set_nonfipsciphers(sslvserver.get_nonfipsciphers())
        __sslvserver.set_ssl2(sslvserver.get_ssl2())
        __sslvserver.set_ssl3(sslvserver.get_ssl3())
        __sslvserver.set_tls1(sslvserver.get_tls1())
        __sslvserver.set_snienable(sslvserver.get_snienable())
        __sslvserver.set_pushenctrigger(sslvserver.get_pushenctrigger())
        return __sslvserver.update_resource(nitro)

    # No unset functionality for now.



from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSSSLCertKeySSLVServerBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSSSLCertKeySSLVServerBinding, self).__init__()
        self.options = {'vserver': '',
                        'crlcheck': '',
                        'ca': '',
                        'vservername': '',
                        'servername': '',
                        'certkey': '',
                        'version': '',
                        'data': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

        self.resourcetype = NSSSLCertKeySSLVServerBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "sslcertkey_sslvserver_binding"

    # Read/write properties
    def set_vserver(self, vserver):
        """
        Specify this option to bind the certificate to an SSL virtual
        server. Note: The default option is -vServer.
        """
        self.options['vserver'] = vserver

    def get_vserver(self):
        """
        Specify this option to bind the certificate to an SSL virtual
        server. Note: The default option is -vServer.
        """
        return self.options['vserver']

    def set_crlcheck(self, crlcheck):
        """
        The rule for use of CRL corresponding to this CA certificate
        during client authentication. If crlCheck is set to Mandatory,
        the system will deny all SSL clients if the CRL is missing,
        expired - NextUpdate date is in the past, or is incomplete with
        remote CRL refresh enabled. If crlCheck is set to optional, the
        system will allow SSL clients in the above error cases.However,
        in any case if the client certificate is revoked in the CRL,
        the SSL client will be denied access.
        Default value: CRLCHECK_OPTIONAL
        """
        self.options['crlcheck'] = crlcheck

    def get_crlcheck(self):
        """
        The rule for use of CRL corresponding to this CA certificate
        during client authentication. If crlCheck is set to Mandatory,
        the system will deny all SSL clients if the CRL is missing,
        expired - NextUpdate date is in the past, or is incomplete with
        remote CRL refresh enabled. If crlCheck is set to optional, the
        system will allow SSL clients in the above error cases.However,
        in any case if the client certificate is revoked in the CRL,
        the SSL client will be denied access.
        Default value: CRLCHECK_OPTIONAL
        """
        return self.options['crlcheck']

    def set_ca(self, ca):
        """
        If this option is specified, it indicates that the
        certificate-key pair being bound to the SSL virtual server is a
        CA certificate. If this option is not specified, the
        certificate-key pair is bound as a normal server certificate.
        Note: In case of a normal server certificate, the
        certificate-key pair should consist of both the certificate and
        the private-key.
        """
        self.options['ca'] = ca

    def get_ca(self):
        """
        If this option is specified, it indicates that the
        certificate-key pair being bound to the SSL virtual server is a
        CA certificate. If this option is not specified, the
        certificate-key pair is bound as a normal server certificate.
        Note: In case of a normal server certificate, the
        certificate-key pair should consist of both the certificate and
        the private-key.
        """
        return self.options['ca']

    def set_vservername(self, vservername):
        """
        The name of the SSL virtual server name to which the
        certificate-key pair needs to be bound.
        Minimum length = 1
        """
        self.options['vservername'] = vservername

    def get_vservername(self):
        """
        The name of the SSL virtual server name to which the
        certificate-key pair needs to be bound.
        Minimum length = 1
        """
        return self.options['vservername']

    def set_servername(self, servername):
        """
        Vserver name to which the certificate key pair is bound.
        """
        self.options['servername'] = servername

    def get_servername(self):
        """
        Vserver name to which the certificate key pair is bound.
        """
        return self.options['servername']

    def set_certkey(self, certkey):
        """
        The object name for the certificate-key pair.
        Minimum length = 1
        """
        self.options['certkey'] = certkey

    def get_certkey(self):
        """
        The object name for the certificate-key pair.
        Minimum length = 1
        """
        return self.options['certkey']

    # Read only properties
    def get_version(self):
        """
        Version.
        """
        return self.options['version']

    def get_data(self):
        """
        Vserver Id.
        """
        return self.options['data']

    @staticmethod
    def get(nitro, sslcertkey):
        """
        Use this API to fetch configured sslcertkey_sslvserver_binding resources of a given name.
        """
        __url = nitro.get_url() + NSSSLCertKeySSLVServerBinding.get_resourcetype() + "/" + sslcertkey.get_certkey()
        __json_bindings = nitro.get(__url).get_response_field(NSSSLCertKeySSLVServerBinding.get_resourcetype())
        __bindings = []
        for json_binding in __json_bindings:
            __bindings.append(NSSSLCertKeySSLVServerBinding(json_binding))
        return __bindings



from nsbaseresource import NSBaseResource


class NSServiceGroupServerBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSServiceGroupServerBinding, self).__init__()
        self.options = {
            'servicegroupname': '',
            'ip': '',
            'servername': '',
            'port': '',
            'weight': '',
            'state': '',
            'serverid': '',
            'hashid': '',
            'svrstate': ''
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

        self.resourcetype = NSServiceGroupServerBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "servicegroup_servicegroupmember_binding"

    # Getters and setters for configurable options
    def set_ip(self, ip):
        """
        The IP address of the server that needs to be binded.
        Default value: 0
        """
        self.options['ip'] = ip

    def get_ip(self):
        """
        The IP address of the server.
        Default value: 0
        """
        return self.options['ip']

    def set_port(self, port):
        """
        The port of the server.
        """
        self.options['port'] = port

    def get_port(self):
        """
        The port of the server that its binded to.
        """
        return self.options['port']

    def set_state(self, state):
        """
        The state of the server.
        """
        self.options['state'] = state

    def get_state(self):
        """
        The state of the server that its binded to.
        """
        return self.options['state']

    def get_svrstate(self):
        """
        The svrstate of the server that its binded to.
        """
        return self.options['svrstate']

    def set_weight(self, weight):
        """
        The weight for the specified service.
        Default value: 0
        Minimum value =  1
        Maximum value =  100
        """
        self.options['weight'] = weight

    def get_weight(self):
        """
        The weight for the specified service.
        Default value: 0
        Minimum value =  1
        Maximum value =  100
        """
        return self.options['weight']

    def set_hashid(self, hashid):
        """
        hash Id.
        Default value: 0
        """
        self.options['hashid'] = hashid

    def get_hashid(self):
        """
        hash Id.
        Default value: 0
        """
        return self.options['hashid']

    def set_serverid(self, serverid):
        """
        server Id.
        Default value: 0
        """
        self.options['serverid'] = serverid

    def get_serverid(self):
        """
        server Id.
        Default value: 0
        """
        return self.options['serverid']

    def set_servername(self, servername):
        """
        The server name to which the service is bound.
        """
        self.options['servername'] = servername

    def get_servername(self):
        """
        The server name to which the service is bound.
        """
        return self.options['servername']

    def set_servicegroupname(self, servicegroupname):
        """
        The name of the service group that is bound.
        """
        self.options['servicegroupname'] = servicegroupname

    def get_servicegroupname(self):
        """
        The name of the service group that is bound.
        """
        return self.options['servicegroupname']

    # Operations methods
    @staticmethod
    def get(nitro, servicegroup_server_binding):
        """
        Use this API to fetch configured servicegroup_server_binding
        """
        __url = nitro.get_url() + NSServiceGroupServerBinding.get_resourcetype() + "/" + servicegroup_server_binding.get_servicegroupname()
        __json_services = nitro.get(__url).get_response_field(NSServiceGroupServerBinding.get_resourcetype())
        __services = []
        for json_service in __json_services:
            __services.append(NSServiceGroupServerBinding(json_service))
        return __services

    @staticmethod
    def get_server(nitro, servicegroup_server_binding):
        """
        Use this API to fetch a server configured to a servicegroup_server_binding
        """
        __servicegroup_server_binding = NSServiceGroupServerBinding()
        __servicegroup_server_binding.set_servicegroupname(servicegroup_server_binding.get_servicegroupname())
        __servicegroup_server_binding.set_servername(servicegroup_server_binding.get_servername())
        __filter = servicegroup_server_binding.get_servicegroupname() + "?filter=servername:" + servicegroup_server_binding.get_servername()
        __servicegroup_server_binding.get_resource(nitro, __filter)
        return __servicegroup_server_binding

    @staticmethod
    def add(nitro, servicegroup_server_binding):
        """
        Use this API to add a server to servergroup through servicegroup_server_binding.
        """
        __servicegroup_server_binding = NSServiceGroupServerBinding()
        __servicegroup_server_binding.set_servername(servicegroup_server_binding.get_servername())
        __servicegroup_server_binding.set_ip(servicegroup_server_binding.get_ip())
        __servicegroup_server_binding.set_port(servicegroup_server_binding.get_port())
        __servicegroup_server_binding.set_servicegroupname(servicegroup_server_binding.get_servicegroupname())
        __servicegroup_server_binding.set_weight(servicegroup_server_binding.get_weight())
        __servicegroup_server_binding.set_serverid(servicegroup_server_binding.get_serverid())
        __servicegroup_server_binding.set_state(servicegroup_server_binding.get_state())
        __servicegroup_server_binding.set_hashid(servicegroup_server_binding.get_hashid())
        return __servicegroup_server_binding.update_resource(nitro)

    @staticmethod
    def delete(nitro, servicegroup_server_binding):
        """
        Use this API to delete a server from servicegroup_server_binding of a given name.
        """
        __servicegroup_server_binding = NSServiceGroupServerBinding()
        __servicegroup_server_binding.set_servicegroupname(servicegroup_server_binding.get_servicegroupname())
        __servicegroup_server_binding.set_servername(servicegroup_server_binding.get_servername())
        __servicegroup_server_binding.set_ip(servicegroup_server_binding.get_ip())
        __servicegroup_server_binding.set_port(servicegroup_server_binding.get_port())
        __filter = servicegroup_server_binding.get_servicegroupname() + "/" + servicegroup_server_binding.get_servername()
        return __servicegroup_server_binding.delete_resource(nitro, __filter)



from nsbaseresource import NSBaseResource
__author__ = 'vlazarenko'


class NSCSVServerRewritePolicyBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSCSVServerRewritePolicyBinding, self).__init__()
        self.options = {'policyname': '',
                        'priority': '',
                        'gotopriorityexpression': '',
                        'bindpoint': '',
                        'invoke': '',
                        'labeltype': '',
                        'labelname': '',
                        'name': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSCSVServerRewritePolicyBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "csvserver_rewritepolicy_binding"

    def set_policyname(self, policyname):
        """
        Bound classic (Compression/Authorization/Filter/Audit) policies to this vserver.

        Default value: 0
        """
        self.options['policyname'] = policyname

    def get_policyname(self):
        """
        Bound classic (Compression/Authorization/Filter/Audit) policies to this vserver.

        Default value: 0
        """
        return self.options['policyname']

    def set_priority(self, priority):
        """
        Priority for the policy.

        Default value: 0
        """
        self.options['priority'] = priority

    def get_priority(self):
        """
        Priority for the policy.

        Default value: 0
        """
        return self.options['priority']

    def set_gotopriorityexpression(self, gotopriorityexpression):
        """
        Expression specifying the priority of the next policy
        which will get evaluated if the current policy rule
        evaluates to TRUE.

        Default value: 0
        """
        self.options['gotopriorityexpression'] = gotopriorityexpression

    def get_gotopriorityexpression(self):
        """
        Expression specifying the priority of the next policy
        which will get evaluated if the current policy rule
        evaluates to TRUE.

        Default value: 0
        """
        return self.options['gotopriorityexpression']

    def set_bindpoint(self, bindpoint):
        """
        The bindpoint to which the policy is bound.

        Default value: 0
        """
        self.options['bindpoint'] = bindpoint

    def get_bindpoint(self):
        """
        The bindpoint to which the policy is bound.

        Default value: 0
        """
        return self.options['bindpoint']

    def set_invoke(self, invoke):
        """
        Invoke flag.

        Default value: 0
        """
        self.options['invoke'] = invoke

    def get_invoke(self):
        """
        Invoke flag.

        Default value: 0
        """
        return self.options['invoke']

    def set_labeltype(self, labeltype):
        """
        The invocation type.

        Default value: 0
        """
        self.options['labeltype'] = labeltype

    def get_labeltype(self):
        """
        The invocation type.

        Default value: 0
        """
        return self.options['labeltype']

    def set_labelname(self, labelname):
        """
        Name of the label invoked.

        Default value: 0
        """
        self.options['labelname'] = labelname

    def get_labelname(self):
        """
        Name of the label invoked.

        Default value: 0
        """
        return self.options['labelname']

    def set_name(self, name):
        """
        The virtual server name (created with the add cs vserver or
        add cr vserver command) for which the content
        switching policy will be set.

        Default value: 0
        Minimum length =  1.
        """
        self.options['name'] = name

    def get_name(self):
        """
        The virtual server name (created with the add cs vserver or
        add cr vserver command) for which the content
        switching policy will be set.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['name']

    # Operations methods
    @staticmethod
    def get(nitro, csvserver_rewritepolicy_binding):
        """
        Use this API to fetch all configured csvserver_rewritepolicy_binding resources.
        """
        __url = nitro.get_url() + NSCSVServerRewritePolicyBinding.get_resourcetype() + "/" + csvserver_rewritepolicy_binding.get_name()
        __json_resources = nitro.get(__url).get_response_field(NSCSVServerRewritePolicyBinding.get_resourcetype())
        __resources = []
        for json_resource in __json_resources:
            __resources.append(NSCSVServerRewritePolicyBinding(json_resource))
        return __resources

    @staticmethod
    def add(nitro, csvserver_rewritepolicy_binding):
        """
        Use this API to add csvserver_rewritepolicy_binding.
        """
        __csvserver_rewritepolicy_binding = NSCSVServerRewritePolicyBinding()
        __csvserver_rewritepolicy_binding.set_name(csvserver_rewritepolicy_binding.get_name())
        __csvserver_rewritepolicy_binding.set_policyname(csvserver_rewritepolicy_binding.get_policyname())
        __csvserver_rewritepolicy_binding.set_priority(csvserver_rewritepolicy_binding.get_priority())
        __csvserver_rewritepolicy_binding.set_gotopriorityexpression(csvserver_rewritepolicy_binding.get_gotopriorityexpression())
        __csvserver_rewritepolicy_binding.set_bindpoint(csvserver_rewritepolicy_binding.get_bindpoint())
        __csvserver_rewritepolicy_binding.set_invoke(csvserver_rewritepolicy_binding.get_invoke())
        __csvserver_rewritepolicy_binding.set_labeltype(csvserver_rewritepolicy_binding.get_labeltype())
        __csvserver_rewritepolicy_binding.set_labelname(csvserver_rewritepolicy_binding.get_labelname())
        return __csvserver_rewritepolicy_binding.update_resource(nitro)

    @staticmethod
    def delete(nitro, csvserver_rewritepolicy_binding):
        """
        Use this API to delete csvserver_rewritepolicy_binding of a given name.
        """
        __csvserver_rewritepolicy_binding = NSCSVServerRewritePolicyBinding()
        __csvserver_rewritepolicy_binding.set_name(csvserver_rewritepolicy_binding.get_name())
        __csvserver_rewritepolicy_binding.set_policyname(csvserver_rewritepolicy_binding.get_policyname())
        __csvserver_rewritepolicy_binding.set_priority(csvserver_rewritepolicy_binding.get_priority())
        __csvserver_rewritepolicy_binding.set_bindpoint(csvserver_rewritepolicy_binding.get_bindpoint())
        nsresponse = __csvserver_rewritepolicy_binding.delete_resource(nitro)
        return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'ndenev@gmail.com'


class SystemGlobalAuthTacacsPolicyBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(SystemGlobalAuthTacacsPolicyBinding, self).__init__()

        self.options = {'priority': '',
                        'builtin': '',
                        'policyname': ''}

        self.resourcetype = SystemGlobalAuthTacacsPolicyBinding.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "systemglobal_authenticationtacacspolicy_binding"

    def set_priority(self, priority):
        """
        The priority of the command policy.
        """
        self.options['priority'] = priority

    def get_priority(self):
        return self.options['priority']

    def set_builtin(self, builtin):
        """
        Indicates that a variable is a built-in (SYSTEM INTERNAL) type.
        Possible values = MODIFIABLE, DELETABLE, IMMUTABLE
        """
        self.options['builtin'] = builtin

    def get_builtin(self):
        return self.options['builtin']

    def set_policyname(self, policyname):
        """
        The name of the command policy.
        """
        self.options['policyname'] = policyname

    def get_policyname(self):
        return self.options['policyname']

    # Operations methods
    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch the configured systemglobal_authenticationtacacspolicy_binding resources.
        """
        __url = nitro.get_url() + SystemGlobalAuthTacacsPolicyBinding.get_resourcetype()
        __json_policybindings = nitro.get(__url).get_response_field(SystemGlobalAuthTacacsPolicyBinding.get_resourcetype())
        __policybindings = []
        for json_policybinding in __json_policybindings:
            __policybindings.append(SystemGlobalAuthTacacsPolicyBinding(json_policybinding))
        return __policybindings

    @staticmethod
    def add(nitro, policybinding):
        """
        Use this API to add systemglobal_authenticationtacacspolicy_binding.
        """
        __policybinding = SystemGlobalAuthTacacsPolicyBinding()
        __policybinding.set_policyname(policybinding.get_policyname())
        __policybinding.set_priority(policybinding.get_priority())
        __policybinding.set_builtin(policybinding.get_builtin())
        return __policybinding.add_resource(nitro)

    @staticmethod
    def delete(nitro, policybinding):
        """
        Use this API to delete systemglobal_authenticationtacacspolicy_binding of a given name.
        """
        __policybinding = SystemGlobalAuthTacacsPolicyBinding()
        __policybinding.set_policyname(policybinding.get_policyname())
        nsresponse = __policybinding.delete_resource(nitro)
        return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSCSPolicy(NSBaseResource):

    # Configuration for content-switching policy resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSCSPolicy, self).__init__()
        self.options = {'policyname': '',
                        'url': '',
                        'rule': '',
                        'domain': '',
                        'vstype': '',
                        'hits': '',
                        'bindhits': '',
                        'labelname': '',
                        'labeltype': '',
                        'target': '',
                        'priority': '',
                        '__count': ''}

        self.resourcetype = NSCSPolicy.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "cspolicy"

    def set_policyname(self, policyname):
        """
        The name of the new content switching policy.

        Default value: 0
        Minimum length =  1.
        """
        self.options['policyname'] = policyname

    def get_policyname(self):
        """
        The name of the new content switching policy.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['policyname']

    def set_url(self, url):
        """
        The URL, with wildcards.
        Specify the string value in this format:
        // [[prefix ] [*]] [.suffix].

        Default value: 0
        Minimum length =  1.
        Maximum length =  208.
        """
        self.options['url'] = url

    def get_url(self):
        """
        The URL, with wildcards.
        Specify the string value in this format:
        // [[prefix ] [*]] [.suffix].

        Default value: 0
        Minimum length =  1.
        Maximum length =  208.
        """
        return self.options['url']

    def set_rule(self, rule):
        """
        The condition for applying this policy.
        Expression logic is as follows:
                - Expression names separated by the logical operators || and &&.
                - Expression names may be grouped using parenthesis.
                - If the expression contains blanks (e.g., between an expression name and a logical operator),
                        then the entire argument must be enclosed in double quotes.
        The following example shows valid expression logic:
                ns_ext_cgi||ns_ext_asp
                "ns_non_get && (ns_header_cookie||ns_header_pragma)".

        Default value: 0
        """
        self.options['rule'] = rule

    def get_rule(self):
        """
        The condition for applying this policy.
        Expression logic is as follows:
                - Expression names separated by the logical operators || and &&.
                - Expression names may be grouped using parenthesis.
                - If the expression contains blanks (e.g., between an expression name and a logical operator),
                        then the entire argument must be enclosed in double quotes.
        The following example shows valid expression logic:
                ns_ext_cgi||ns_ext_asp
                "ns_non_get && (ns_header_cookie||ns_header_pragma)".

        Default value: 0
        """
        return self.options['rule']

    def set_domain(self, domain):
        """
        The domain name. The string value can range to 63 characters.

        Default value: 0
        Minimum length =  1.
        """
        self.options['domain'] = domain

    def get_domain(self):
        """
        The domain name. The string value can range to 63 characters.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['domain']

    def get_vstype(self):
        """
        Virtual server type.

        Default value: 0
        """
        return self.options['vstype']

    def get_hits(self):
        """
        Total number of hits.

        Default value: 0
        """
        return self.options['hits']

    def get_bindhits(self):
        """
        Total number of hits.

        Default value: 0
        """
        return self.options['bindhits']

    def get_labelname(self):
        """
        Name of the label invoked.

        Default value: 0
        """
        return self.options['labelname']

    def get_labeltype(self):
        """
        The invocation type.

        Default value: 0
        """
        return self.options['labeltype']

    def get_target(self):
        """
        Target flag.

        Default value: 0
        """
        return self.options['target']

    def get_priority(self):
        """
        Priority of bound policy.

        Default value: 0
        """
        return self.options['priority']

    # Operations methods
    @staticmethod
    def get(nitro, cspolicy):
        """
        Use this API to fetch cspolicy resource of given name.
        """
        __cspolicy = NSCSPolicy()
        __cspolicy.set_policyname(cspolicy.get_policyname())
        __cspolicy.get_resource(nitro)
        return __cspolicy

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured cspolicy resources.
        """
        __url = nitro.get_url() + NSCSPolicy.get_resourcetype()
        __json_cspolicies = nitro.get(__url).get_response_field(NSCSPolicy.get_resourcetype())
        __cspolicies = []
        for json_cspolicy in __json_cspolicies:
            __cspolicies.append(NSCSPolicy(json_cspolicy))
        return __cspolicies

    @staticmethod
    def add(nitro, cspolicy):
        """
        Use this API to add cspolicy.
        """
        __cspolicy = NSCSPolicy()
        __cspolicy.set_policyname(cspolicy.get_policyname())
        __cspolicy.set_url(cspolicy.get_url())
        __cspolicy.set_rule(cspolicy.get_rule())
        __cspolicy.set_domain(cspolicy.get_domain())
        return __cspolicy.add_resource(nitro)

    @staticmethod
    def delete(nitro, cspolicy):
        """
        Use this API to delete cspolicy of a given name.
        """
        __cspolicy = NSCSPolicy()
        __cspolicy.set_policyname(cspolicy.get_policyname())
        nsresponse = __cspolicy.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, cspolicy):
        """
        Use this API to update a cspolicy of a given name.
        """
        __cspolicy = NSCSPolicy()
        __cspolicy.set_policyname(cspolicy.get_policyname())
        __cspolicy.set_url(cspolicy.get_url())
        __cspolicy.set_rule(cspolicy.get_rule())
        __cspolicy.set_domain(cspolicy.get_domain())
        return __cspolicy.update_resource(nitro)


    # No unset functionality for now.



from nsbaseresource import NSBaseResource

__author__ = 'ivanxx@gmail.com'


class SystemUser(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(SystemUser, self).__init__()

        self.options = {'username': '',
                        'password': '',
                        'priority': ''}

        self.resourcetype = SystemUser.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "systemuser"

    def set_username(self, username):
        self.options['username'] = username

    def get_username(self):
        return self.options['username']

    def set_password(self, password):
        self.options['password'] = password

    def get_password(self):
        return self.options['password']

    def set_priority(self, password):
        # The priority, when submitted to the Nitro API, is ignored by the Nitro API
        # and it is always zero
        self.options['priority'] = password

    def get_priority(self):
        # The priority, when retrieve from the Nitro API, is always zero
        return self.options['priority']

    @staticmethod
    def add(nitro, systemuser):
        __systemuser = SystemUser()
        __systemuser.set_username(systemuser.get_username())
        __systemuser.set_password(systemuser.get_password())
        return __systemuser.add_resource(nitro)

    @staticmethod
    def get(nitro, systemuser):
        __systemuser = SystemUser()
        __systemuser.set_username(systemuser.get_username())
        __systemuser.get_resource(nitro, object_name=__systemuser.get_username())
        return __systemuser

    @staticmethod
    def update(nitro, systemuser):
        __systemuser = SystemUser()
        __systemuser.set_username(systemuser.get_username())
        __systemuser.set_password(systemuser.get_password())
        return __systemuser.update_resource(nitro)

    @staticmethod
    def delete(nitro, systemuser):
        __systemuser = SystemUser()
        __systemuser.set_username(systemuser.get_username())
        return __systemuser.delete_resource(nitro)

    @staticmethod
    def get_all(nitro):
        __url = nitro.get_url() + SystemUser.get_resourcetype()
        __json_systemusers = nitro.get(__url).get_response_field(SystemUser.get_resourcetype())

        return [SystemUser(u) for u in __json_systemusers]



from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSLBVServerCSVserverBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSLBVServerCSVserverBinding, self).__init__()
        self.options = {'priority': '',
                        'policyname': '',
                        'name': '',
                        'cachevserver': '',
                        'cachetype': '',
                        'hits': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSLBVServerCSVserverBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        """
        Binding object showing the csvserver that can be bound to lbvserver.
        """
        return "lbvserver_csvserver_binding"

    # Read/write properties
    def set_priority(self, priority):
        """
        Priority.
        """
        self.options['priority'] = priority

    def get_priority(self):
        """
        Priority.
        """
        return self.options['priority']

    def set_policyname(self, policyname):
        """
        Name of the policy bound to the LB vserver.
        """
        self.options['policyname'] = policyname

    def get_policyname(self):
        """
        Name of the policy bound to the LB vserver.
        """
        return self.options['policyname']

    def set_name(self, name):
        """
        The virtual server name to which the service is bound.
        Minimum length = 1
        """
        self.options['name'] = name

    def get_name(self):
        """
        The virtual server name to which the service is bound.
        Minimum length = 1
        """
        return self.options['name']

    def set_cachevserver(self, cachevserver):
        """
        Cache virtual server.
        """
        self.options['cachevserver'] = cachevserver

    def get_cachevserver(self):
        """
        Cache virtual server.
        """
        return self.options['cachevserver']

    # Read only properties
    def get_cachetype(self):
        """
        Cache type.
        """
        return self.options['cachetype']

    def get_hits(self):
        """
        Number of hits.
        """
        return self.options['hits']

    @staticmethod
    def get(nitro, lbvservercsvserverbinding):
        """
        Use this API to fetch lb vserver cs vserver binding resource of given name.
        """
        __url = nitro.get_url() + NSLBVServerCSVserverBinding.get_resourcetype() + "/" + lbvservercsvserverbinding.get_name()
        __json_csvservers = nitro.get(__url).get_response_field(NSLBVServerCSVserverBinding.get_resourcetype())
        __csvservers = []
        for json_csvserver in __json_csvservers:
            __csvservers.append(NSLBVServerCSVserverBinding(json_csvserver))
        return __csvservers



from nsbaseresource import NSBaseResource

__author__ = 'Aleksandar Topuzovic'


class NSTCPProfile(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSTCPProfile, self).__init__()

        self.options = {
            # Read-write values
            'name': '',
            'ws': '',
            'sack': '',
            'wsval': '',
            'nagle': '',
            'ackonpush': '',
            'mss': '',
            'maxburst': '',
            'initialcwnd': '',
            'delayedack': '',
            'oooqsize': '',
            'maxpktpermss': '',
            'pktperretx': '',
            'minrto': '',
            'slowstartincr': '',
            'buffersize': '',

            # Readonly values
            'refcnt': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

        self.resourcetype = NSTCPProfile.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "nstcpprofile"

    # Read-write
    def get_name(self):
        """
        Name of the TCP profile.
        Minimum length = 1
        Maximum length = 127
        """
        return self.options['name']

    def set_name(self, name):
        """
        Name of the TCP profile.
        Minimum length = 1
        Maximum length = 127
        """
        self.options['name'] = name

    def get_ws(self):
        """
        The state of WS.
        Default value: DISABLED
        """
        return self.options['ws']

    def set_ws(self, ws):
        """
        The state of WS.
        Default value: DISABLED
        """
        self.options['ws'] = ws

    def get_sack(self):
        """
        The state of SACK.
        Default value: DISABLED
        """
        return self.options['sack']

    def set_sack(self, sack):
        """
        The state of SACK.
        Default value: DISABLED
        """
        self.options['sack'] = sack

    def get_wsval(self):
        """
        Window Scaling Factor used.
        Default value: 4
        Minimum value = 0
        Maximum value = 8
        """
        return self.options['wsval']

    def set_wsval(self, wsval):
        """
        Window Scaling Factor used.
        Default value: 4
        Minimum value = 0
        Maximum value = 8
        """
        self.options['wsval'] = wsval

    def get_nagle(self):
        """
        Whether to enable Nagle's algorithm on connections.
        Default value: DISABLED
        """
        return self.options['nagle']

    def set_nagle(self, nagle):
        """
        Whether to enable Nagle's algorithm on connections.
        Default value: DISABLED
        """
        self.options['nagle'] = nagle

    def get_ackonpush(self):
        """
        Enable/disable immediate ACK on PUSH packet.
        Default value: ENABLED
        """
        return self.options['ackonpush']

    def set_ackonpush(self, ackonpush):
        """
        Enable/disable immediate ACK on PUSH packet.
        Default value: ENABLED
        """
        self.options['ackonpush'] = ackonpush

    def get_mss(self):
        """
        Set Maximum Segment Size(MSS) to use for TCP Connection (0 forces use of global setting).
        Default value: 0
        Minimum value = 0
        Maximum value = 1460
        """
        return self.options['mss']

    def set_mss(self, mss):
        """
        Set Maximum Segment Size(MSS) to use for TCP Connection (0 forces use of global setting).
        Default value: 0
        Minimum value = 0
        Maximum value = 1460
        """
        self.options['mss'] = mss

    def get_maxburst(self):
        """
        Max-Burst Factor used.
        Default value: 6
        Minimum value = 1
        Maximum value = 255
        """
        return self.options['maxburst']

    def set_maxburst(self, maxburst):
        """
        Max-Burst Factor used.
        Default value: 6
        Minimum value = 1
        Maximum value = 255
        """
        self.options['maxburst'] = maxburst

    def get_initialcwnd(self):
        """
        Intial value of TCP cwnd used.
        Default value: 4
        Minimum value = 1
        Maximum value = 44
        """
        return self.options['initialcwnd']

    def set_initialcwnd(self, initialcwnd):
        """
        Intial value of TCP cwnd used.
        Default value: 4
        Minimum value = 1
        Maximum value = 44
        """
        self.options['initialcwnd'] = initialcwnd

    def get_delayedack(self):
        """
        Delayed acknowledgement timeout (in millisec).
        Default value: 200
        Minimum value = 10
        Maximum value = 300
        """
        return self.options['delayedack']

    def set_delayedack(self, delayedack):
        """
        Delayed acknowledgement timeout (in millisec).
        Default value: 200
        Minimum value = 10
        Maximum value = 300
        """
        self.options['delayedack'] = delayedack

    def get_oooqsize(self):
        """
        Maximum size of out-of-order packet queue (0 means infinite).
        Default value: 64
        Minimum value = 0
        Maximum value = 65535
        """
        return self.options['oooqsize']

    def set_oooqsize(self, oooqsize):
        """
        Maximum size of out-of-order packet queue (0 means infinite).
        Default value: 64
        Minimum value = 0
        Maximum value = 65535
        """
        self.options['oooqsize'] = oooqsize

    def get_maxpktpermss(self):
        """
        Enable packet count based congestion control by setting to non zero value.
        Default value: 0
        Minimum value = 0
        Maximum value = 512
        """
        return self.options['maxpktpermss']

    def set_maxpktpermss(self, maxpktpermss):
        """
        Enable packet count based congestion control by setting to non zero value.
        Default value: 0
        Minimum value = 0
        Maximum value = 512
        """
        self.options['maxpktpermss'] = maxpktpermss

    def get_pktperretx(self):
        """
        Set value for maximum number packets per retransmission.
        Default value: 1
        Minimum value = 1
        Maximum value = 512
        """
        return self.options['pktperretx']

    def set_pktperretx(self, pktperretx):
        """
        Set value for maximum number packets per retransmission.
        Default value: 1
        Minimum value = 1
        Maximum value = 512
        """
        self.options['pktperretx'] = pktperretx

    def get_minrto(self):
        """
        Set minimum limit on TCP RTO (in millisec).
        Default value: 1000
        Minimum value = 10
        Maximum value = 64000
        """
        return self.options['minrto']

    def set_minrto(self, minrto):
        """
        Set minimum limit on TCP RTO (in millisec).
        Default value: 1000
        Minimum value = 10
        Maximum value = 64000
        """
        self.options['minrto'] = minrto

    def get_slowstartincr(self):
        """
        Set TCP slowstart increment factor.
        Default value: 2
        Minimum value = 1
        Maximum value = 100
        """
        return self.options['slowstartincr']

    def set_slowstartincr(self, slowstartincr):
        """
        Set TCP slowstart increment factor.
        Default value: 2
        Minimum value = 1
        Maximum value = 100
        """
        self.options['slowstartincr'] = slowstartincr

    def get_buffersize(self):
        """
        Set TCP buffer size.
        Default value: 8190
        Minimum value = 8190
        Maximum value = 4194304
        """
        return self.options['buffersize']

    def set_buffersize(self, buffersize):
        """
        Set TCP buffer size.
        Default value: 8190
        Minimum value = 8190
        Maximum value = 4194304
        """
        self.options['buffersize'] = buffersize

    # Read-only
    def get_refcnt(self):
        """
        Number of entities using this profile
        """
        return self.options['refcnt']

    # Operations methods

    @staticmethod
    def get(nitro, tcpprofile):
        """
        Use this API to fetch service resource of given name.
        """
        __tcpprofile = NSTCPProfile()
        __tcpprofile.set_name(tcpprofile.get_name())
        __tcpprofile.get_resource(nitro)
        return __tcpprofile

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured service resources.
        """
        __url = nitro.get_url() + NSTCPProfile.get_resourcetype()
        __json_tcpprofiles = nitro.get(__url).get_response_field(NSTCPProfile.get_resourcetype())
        __tcpprofiles = []
        for json_service in __json_tcpprofiles:
            __tcpprofiles.append(NSTCPProfile(json_service))
        return __tcpprofiles

    @staticmethod
    def add(nitro, tcpprofile):
        """
        Use this API to add a tcp profile.
        """
        __tcpprofile = NSTCPProfile()
        __tcpprofile.set_name(tcpprofile.get_name())
        __tcpprofile.set_ws(tcpprofile.get_ws())
        __tcpprofile.set_sack(tcpprofile.get_sack())
        __tcpprofile.set_wsval(tcpprofile.get_wsval())
        __tcpprofile.set_nagle(tcpprofile.get_nagle())
        __tcpprofile.set_ackonpush(tcpprofile.get_ackonpush())
        __tcpprofile.set_mss(tcpprofile.get_mss())
        __tcpprofile.set_maxburst(tcpprofile.get_maxburst())
        __tcpprofile.set_initialcwnd(tcpprofile.get_initialcwnd())
        __tcpprofile.set_delayedack(tcpprofile.get_delayedack())
        __tcpprofile.set_oooqsize(tcpprofile.get_oooqsize())
        __tcpprofile.set_maxpktpermss(tcpprofile.get_maxpktpermss())
        __tcpprofile.set_pktperretx(tcpprofile.get_pktperretx())
        __tcpprofile.set_minrto(tcpprofile.get_minrto())
        __tcpprofile.set_slowstartincr(tcpprofile.get_slowstartincr())
        __tcpprofile.set_buffersize(tcpprofile.get_buffersize())
        return __tcpprofile.add_resource(nitro)

    @staticmethod
    def update(nitro, tcpprofile):
        """
        Use this API to update a tcp profile.
        """
        __tcpprofile = NSTCPProfile()
        __tcpprofile.set_name(tcpprofile.get_name())
        __tcpprofile.set_ws(tcpprofile.get_ws())
        __tcpprofile.set_sack(tcpprofile.get_sack())
        __tcpprofile.set_wsval(tcpprofile.get_wsval())
        __tcpprofile.set_nagle(tcpprofile.get_nagle())
        __tcpprofile.set_ackonpush(tcpprofile.get_ackonpush())
        __tcpprofile.set_mss(tcpprofile.get_mss())
        __tcpprofile.set_maxburst(tcpprofile.get_maxburst())
        __tcpprofile.set_initialcwnd(tcpprofile.get_initialcwnd())
        __tcpprofile.set_delayedack(tcpprofile.get_delayedack())
        __tcpprofile.set_oooqsize(tcpprofile.get_oooqsize())
        __tcpprofile.set_maxpktpermss(tcpprofile.get_maxpktpermss())
        __tcpprofile.set_pktperretx(tcpprofile.get_pktperretx())
        __tcpprofile.set_minrto(tcpprofile.get_minrto())
        __tcpprofile.set_slowstartincr(tcpprofile.get_slowstartincr())
        __tcpprofile.set_buffersize(tcpprofile.get_buffersize())
        return __tcpprofile.update_resource(nitro)

    @staticmethod
    def delete(nitro, tcpprofile):
        """
        Use this API to delete a tcpprofile of a given name.
        """
        __tcpprofile = NSTCPProfile()
        __tcpprofile.set_name(tcpprofile.get_name())
        nsresponse = __tcpprofile.delete_resource(nitro)
        return nsresponse



from nsbaseresource import NSBaseResource


class NSHANode(NSBaseResource):

    # Configuration for HA node resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSHANode, self).__init__()
        self.options = {'id': '',
                        'ipaddress': '',
                        'inc': '',
                        'hastatus': '',
                        'hasync': '',
                        'haprop': '',
                        'hellointerval': '',
                        'deadinterval': '',
                        'failsafe': '',
                        'name': '',
                        'flags': '',
                        'state': '',
                        'enaifaces': '',
                        'disifaces': '',
                        'hamonifaces': '',
                        'pfifaces': '',
                        'ifaces': '',
                        'network': '',
                        'netmask': '',
                        'ssl2': '',
                        'masterstatetime': '',
                        'routemonitor': ''}

        self.resourcetype = NSHANode.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "hanode"

    def set_id(self, myid):
        self.options['id'] = myid

    def get_id(self):
        return self.options['id']

    def set_ipaddress(self, ipaddress):
        self.options['ipaddress'] = ipaddress

    def get_ipaddress(self):
        return self.options['ipaddress']

    def set_inc(self, inc):
        self.options['inc'] = inc

    def get_inc(self):
        return self.options['inc']

    def set_hastatus(self, hastatus):
        self.options['hastatus'] = hastatus

    def get_hastatus(self):
        return self.options['hastatus']

    def set_hasync(self, hasync):
        self.options['hasync'] = hasync

    def get_hasync(self):
        return self.options['hasync']

    def set_haprop(self, haprop):
        self.options['haprop'] = haprop

    def get_haprop(self):
        return self.options['haprop']

    def set_hellointerval(self, hellointerval):
        self.options['hellointerval'] = hellointerval

    def get_hellointerval(self):
        return self.options['hellointerval']

    def set_deadinterval(self, deadinterval):
        self.options['deadinterval'] = deadinterval

    def get_deadinterval(self):
        return self.options['deadinterval']

    def set_failsafe(self, failsafe):
        self.options['failsafe'] = failsafe

    def get_failsafe(self):
        return self.options['failsafe']

    def get_name(self):
        return self.options['name']

    def get_flags(self):
        return self.options['flags']

    def get_state(self):
        return self.options['state']

    def get_enaifaces(self):
        return self.options['enaifaces']

    def get_disifaces(self):
        return self.options['disifaces']

    def get_hamonifaces(self):
        return self.options['hamonifaces']

    def get_pfifaces(self):
        return self.options['pfifaces']

    def get_ifaces(self):
        return self.options['ifaces']

    def get_network(self):
        return self.options['network']

    def get_netmask(self):
        return self.options['netmask']

    def get_ssl2(self):
        return self.options['ssl2']

    def get_masterstatetime(self):
        return self.options['masterstatetime']

    def get_routemonitor(self):
        return self.options['routemonitor']

    # Operations methods
    @staticmethod
    def get(nitro, hanode):
        """
        Use this API to fetch hanode resource of given name.
        """
        __hanode = NSHANode()
        __hanode.get_resource(nitro, hanode.get_id())
        return __hanode

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured hanode resources.
        """
        __url = nitro.get_url() + NSHANode.get_resourcetype()
        __json_hanodes = nitro.get(__url).get_response_field(NSHANode.get_resourcetype())
        __hanodes = []
        for json_hanode in __json_hanodes:
            __hanodes.append(NSHANode(json_hanode))
        return __hanodes

    @staticmethod
    def add(nitro, hanode):
        """
        Use this API to add hanode.
        """
        __hanode = NSHANode()
        __hanode.set_id(hanode.get_id())
        __hanode.set_ipaddress(hanode.get_ipaddress())
        __hanode.set_inc(hanode.get_inc())
        return __hanode.add_resource(nitro)

    @staticmethod
    def delete(nitro, hanode):
        """
        Use this API to delete hanode of a given name.
        """
        __hanode = NSHANode()
        nsresponse = __hanode.delete_resource(nitro, hanode.get_id())
        return nsresponse

    @staticmethod
    def update(nitro, hanode):
        """
        Use this API to update a hanode of a given name.
        """
        __hanode = NSHANode()
        __hanode.set_id(hanode.get_id())
        __hanode.set_hastatus(hanode.get_hastatus())
        __hanode.set_hasync(hanode.get_hasync())
        __hanode.set_haprop(hanode.get_haprop())
        __hanode.set_hellointerval(hanode.get_hellointerval())
        __hanode.set_deadinterval(hanode.get_deadinterval())
        __hanode.set_failsafe(hanode.get_failsafe())
        return __hanode.update_resource(nitro)


    # No unset functionality for now.



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSSSLVServerSSLCertKeyBinding(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSSSLVServerSSLCertKeyBinding, self).__init__()

        self.options = {'certkeyname': '',
                        'cleartextport': '',
                        'crlcheck': '',
                        'ocspcheck': '',
                        'vservername': '',
                        'ca': '',
                        'snicert': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSSSLVServerSSLCertKeyBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "sslvserver_sslcertkey_binding"

    def set_certkeyname(self, certkeyname):
        self.options['certkeyname'] = certkeyname

    def get_certkeyname(self):
        return self.options['certkeyname']

    def set_cleartextport(self, cleartextport):
        self.options['cleartextport'] = cleartextport

    def get_cleartextport(self):
        return self.options['cleartextport']

    def set_crlcheck(self, crlcheck):
        self.options['crlcheck'] = crlcheck

    def get_crlcheck(self):
        return self.options['crlcheck']

    def set_ocspcheck(self, ocspcheck):
        self.options['ocspcheck'] = ocspcheck

    def get_ocspcheck(self):
        return self.options['ocspcheck']

    def set_vservername(self, vservername):
        self.options['vservername'] = vservername

    def get_vservername(self):
        return self.options['vservername']

    def set_ca(self, ca):
        self.options['ca'] = ca

    def get_ca(self):
        return self.options['ca']

    def set_snicert(self, snicert):
        self.options['snicert'] = snicert

    def get_snicert(self):
        return self.options['snicert']

    @staticmethod
    def get(nitro, resource):
        """
        Use this API to fetch SSL certkey resource of given name.
        """
        __url = nitro.get_url() + NSSSLVServerSSLCertKeyBinding.get_resourcetype() + "/" + resource.get_vservername()
        __json_resources = nitro.get(__url).get_response_field(NSSSLVServerSSLCertKeyBinding.get_resourcetype())
        __resources = []
        for json_resource in __json_resources:
            __resources.append(NSSSLVServerSSLCertKeyBinding(json_resource))
        return __resources

    @staticmethod
    def add(nitro, resource):
        """
        Use this API to add resource.
        """
        __resource = NSSSLVServerSSLCertKeyBinding()
        __resource.set_vservername(resource.get_vservername())
        __resource.set_certkeyname(resource.get_certkeyname())
        __resource.set_ca(resource.get_ca())
        __resource.set_crlcheck(resource.get_crlcheck())
        __resource.set_snicert(resource.get_snicert())
        __resource.set_ocspcheck(resource.get_ocspcheck())
        return __resource.add_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        """
        Use this API to delete server of a given name.
        """
        __resource = NSSSLVServerSSLCertKeyBinding()
        __resource.set_vservername(resource.get_vservername())
        __resource.set_certkeyname(resource.get_certkeyname())
        __resource.set_ca(resource.get_ca())
        __resource.set_crlcheck(resource.get_crlcheck())
        __resource.set_snicert(resource.get_snicert())
        nsresponse = __resource.delete_resource(nitro)
        return nsresponse



from nsbaseresource import NSBaseResource
from nsconfig import NSConfig
from nscspolicy import NSCSPolicy
from nscsvserver import NSCSVServer
from nscsvservercspolicybinding import NSCSVServerCSPolicyBinding
from nscsvserverresponderpolicybinding import NSCSVServerResponderPolicyBinding
from nscsvserverrewritepolicybinding import NSCSVServerRewritePolicyBinding
from nslbmonitor import NSLBMonitor
from nslbvserver import NSLBVServer
from nslbvserverservicebinding import NSLBVServerServiceBinding
from nslbvserverservicegroupbinding import NSLBVServerServiceGroupBinding
from nslbvservercsvserverbinding import NSLBVServerCSVserverBinding
from nsresponderaction import NSResponderAction
from nsresponderpolicy import NSResponderPolicy
from nsresponderpolicylabel import NSResponderPolicyLabel
from nsresponderpolicylabelbinding import NSResponderPolicyLabelBinding
from nsresponderpolicycsvserverbinding import NSResponderPolicyCSVServerBinding
from nsrewritepolicy import NSRewritePolicy
from nsrewritepolicycsvserverbinding import NSRewritePolicyCSVServerBinding
from nsrunningconfig import NSRunningConfig
from nssavedconfig import NSSavedConfig
from nsservice import NSService
from nsserver import NSServer
from nsservicegroup import NSServiceGroup
from nsservicegroupserverbinding import NSServiceGroupServerBinding
from nsservicelbmonitorbinding import NSServiceLBMonitorBinding
from nssslaction import NSSSLAction
from nssslcertkey import NSSSLCertKey
from nssslcertkeysslvserverbinding import NSSSLCertKeySSLVServerBinding
from nssslcertlink import NSSSLCertLink
from nssslpolicy import NSSSLPolicy
from nssslservice import NSSSLService
from nssslservicesslpolicybinding import NSSSLServiceSSLPolicyBinding
from nssslvserver import NSSSLVServer
from nssslvserversslcertkeybinding import NSSSLVServerSSLCertKeyBinding
from nssslvserversslpolicybinding import NSSSLVServerSSLPolicyBinding
from nstcpprofile import NSTCPProfile
from nshanode import NSHANode
from nshardware import NSHardware
from nshostname import NSHostname
from nshttpprofile import NSHTTPProfile
from nsip import NSIP
from nsiptunnel import NSIPTunnel
from nsroute import NSRoute
from nsversion import NSVersion
from nsvlan import NSVLAN
from nsvlaninterfacebinding import NSVLANInterfaceBinding
from nsvlannsipbinding import NSVLANNSIPBinding
from nsvserver import NSVServer
from nsfeature import NSFeature
from nsrewriteaction import NSRewriteAction
from nslbmonitorservicebinding import NSLBMonitorServiceBinding
from nssystemcmdpolicy import NSSystemCMDPolicy
from nsacl import NSAcl
from nsacls import NSAcls
from nspbr import NSPbr
from nspbrs import NSPbrs
from authenticationtacacsaction import AuthTacacsAction
from authenticationtacacspolicy import AuthTacacsPolicy
from snmpcommunity import SNMPCommunity
from snmpmanager import SNMPManager
from systemuser import SystemUser
from systemglobalauthenticationtacacspolicybinding import SystemGlobalAuthTacacsPolicyBinding

__all__ = ['NSBaseResource',
           'NSConfig',
           'NSCSPolicy',
           'NSCSVServer',
           'NSCSVServerCSPolicyBinding',
           'NSCSVServerResponderPolicyBinding',
           'NSCSVServerRewritePolicyBinding',
           'NSLBMonitor',
           'NSLBVServer',
           'NSLBVServerServiceBinding',
           'NSLBVServerServiceGroupBinding',
           'NSLBVServerCSVserverBinding',
           'NSResponderAction',
           'NSResponderPolicy',
           'NSResponderPolicyLabel',
           'NSResponderPolicyLabelBinding',
           'NSResponderPolicyCSVServerBinding',
           'NSRewritePolicy',
           'NSRewritePolicyCSVServerBinding',
           'NSRunningConfig',
           'NSSavedConfig',
           'NSServer',
           'NSService',
           'NSServiceGroup',
           'NSServiceGroupServerBinding',
           'NSServiceLBMonitorBinding',
           'NSSSLAction',
           'NSSSLCertKey',
           'NSSSLCertKeySSLVServerBinding',
           'NSSSLCertLink',
           'NSSSLService',
           'NSSSLServiceSSLPolicyBinding',
           'NSSSLPolicy',
           'NSSSLVServer',
           'NSSSLVServerSSLCertKeyBinding',
           'NSSSLVServerSSLPolicyBinding',
           'NSTCPProfile',
           'NSHANode',
           'NSHardware',
           'NSHostname',
           'NSHTTPProfile',
           'NSIP',
           'NSIPTunnel',
           'NSRoute',
           'NSVersion',
           'NSVLAN',
           'NSVLANInterfaceBinding',
           'NSVLANNSIPBinding',
           'NSVServer',
           'NSFeature',
           'NSRewriteAction',
           'NSLBMonitorServiceBinding',
           'NSSystemCMDPolicy',
           'NSAcl',
           'NSAcls',
           'NSPbr',
           'NSPbrs',
           'AuthTacacsAction',
           'AuthTacacsPolicy',
           'SNMPCommunity',
           'SNMPManager',
           'SystemGlobalAuthTacacsPolicyBinding',
           'SystemUser'
           ]



from nsbaseresource import NSBaseResource
__author__ = 'timl'


class NSResponderPolicyLabelBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSResponderPolicyLabelBinding, self).__init__()
        self.options = {
            'labelname': '',
            'policyname': '',
            'priority': '',
            'gotopriorityexpression': '',
            'flags': '',
            'labeltype': '',
            'invoke_labelname': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSResponderPolicyLabelBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "responderpolicylabel_responderpolicy_binding"

    def set_labelname(self, labelname):
        """
        Responder policy label.

        Default value: 0
        """
        self.options['labelname'] = labelname

    def get_labelname(self):
        """
        Responder policy label.

        Default value: 0
        """
        return self.options['labelname']

    def set_policyname(self, policyname):
        """
        Bound responder policies to this responder policy label.

        Default value: 0
        """
        self.options['policyname'] = policyname

    def get_policyname(self):
        """
        Bound responder policies to this responder policy label.

        Default value: 0
        """
        return self.options['policyname']

    def set_priority(self, priority):
        """
        Priority for the policy.

        Default value: 0
        """
        self.options['priority'] = priority

    def get_priority(self):
        """
        Priority for the policy.

        Default value: 0
        """
        return self.options['priority']

    def set_gotopriorityexpression(self, gotopriorityexpression):
        """
        Expression specifying the priority of the next policy which will get evaluated if the current policy rule
        evaluates to TRUE.

        Default value: 0
        """
        self.options['gotopriorityexpression'] = gotopriorityexpression

    def get_gotopriorityexpression(self):
        """
        Expression specifying the priority of the next policy which will get evaluated if the current policy rule
        evaluates to TRUE.

        Default value: 0
        """
        return self.options['gotopriorityexpression']

    def set_flags(self, flags):
        """
        Bind flags.

        Default value: 0
        """
        self.options['flags'] = flags

    def get_flags(self):
        """
        Bind flags.

        Default value: 0
        """
        return self.options['flags']

    def set_labeltype(self, labeltype):
        """
        The invocation type.

        Default value: 0
        """
        self.options['labeltype'] = labeltype

    def get_labeltype(self):
        """
        The invocation type.

        Default value: 0
        """
        return self.options['labeltype']

    def set_invoke_labelname(self, invoke_labelname):
        """
        Name of the label invoked.

        Default value: 0
        """
        self.options['invoke_labelname'] = invoke_labelname

    def get_invoke_labelname(self):
        """
        Name of the label invoked.

        Default value: 0
        """
        return self.options['invoke_labelname']

    # Operations methods
    @staticmethod
    def get(nitro, responderpolicylabel_binding):
        """
        Use this API to fetch all configured responderpolicylabel_binding resources.
        """
        __url = nitro.get_url() + NSResponderPolicyLabelBinding.get_resourcetype() + "/" + responderpolicylabel_binding.get_labelname()
        __json_resources = nitro.get(__url).get_response_field(NSResponderPolicyLabelBinding.get_resourcetype())
        __resources = []
        for json_resource in __json_resources:
            __resources.append(NSResponderPolicyLabelBinding(json_resource))
        return __resources

    @staticmethod
    def add(nitro, responderpolicylabel_binding):
        """
        Use this API to add responderpolicylabel_binding.
        """
        __responderpolicylabel_binding = NSResponderPolicyLabelBinding()
        __responderpolicylabel_binding.set_labelname(responderpolicylabel_binding.get_labelname())
        __responderpolicylabel_binding.set_policyname(responderpolicylabel_binding.get_policyname())
        __responderpolicylabel_binding.set_priority(responderpolicylabel_binding.get_priority())
        __responderpolicylabel_binding.set_gotopriorityexpression(responderpolicylabel_binding.get_gotopriorityexpression())
        __responderpolicylabel_binding.set_flags(responderpolicylabel_binding.get_flags())
        __responderpolicylabel_binding.set_labeltype(responderpolicylabel_binding.get_labeltype())
        __responderpolicylabel_binding.set_invoke_labelname(responderpolicylabel_binding.get_invoke_labelname())
        return __responderpolicylabel_binding.update_resource(nitro)

    @staticmethod
    def delete(nitro, responderpolicylabel_binding):
        """
        Use this API to delete responderpolicylabel_binding of a given name.
        """
        __responderpolicylabel_binding = NSResponderPolicyLabelBinding()
        __name = responderpolicylabel_binding.get_labelname()
        __responderpolicylabel_binding.set_labelname(__name)
        __responderpolicylabel_binding.set_policyname(responderpolicylabel_binding.get_policyname())
        nsresponse = __responderpolicylabel_binding.delete_resource(nitro, __name)
        return nsresponse



#!/usr/bin/env python
#
# Netscaler NITRO controller
#
# import the necessary libraries
import argparse
import getpass
import sys
import nsnitro
from nsnitro.nsnitro import *
from nsnitro.nsresources.nsconfig import NSConfig
from nsnitro.nsutil import *
from nsnitro.nsresources.nslbvserver import NSLBVServer
from nsnitro.nsresources.nsservice import NSService
from nsnitro.nsresources.nsserver import NSServer
from nsnitro.nsresources.nscsvserver import NSCSVServer
from nsnitro.nsresources.nsacl import NSAcl
from nsnitro.nsresources.nsacls import NSAcls
from nsnitro.nsresources.nshanode import NSHANode
from nsnitro.nsresources.nsservicegroupserverbinding import NSServiceGroupServerBinding
from nsnitro.nsresources.nslbvserverservicebinding import NSLBVServerServiceBinding


def action_getprimarylb(args):
        ha_node = NSHANode()
        for node in ha_node.get_all(nitro):
                state = node.get_state().lower()
                if state == 'primary':
                        print(node.get_ipaddress())
                        break

def action_get(args, nitro):
        if args.object == "csvserver":
                csvserver = NSCSVServer()
                csvserver.set_name(args.object_name)
                csvserver = NSCSVServer.get(nitro, csvserver)
                print "--- CS vserver: " + csvserver.get_name() + " ---"
                for k in sorted(csvserver.options.iterkeys(), key=lambda k: k):
                        print "\t%s: %s" % (k, csvserver.options[k])

        if args.object == "lbvserver":
                lbvserver = NSLBVServer()
                lbvserver.set_name(args.object_name)
                lbvserver = NSLBVServer.get(nitro, lbvserver)
                print "--- LB vserver: " + lbvserver.get_name() + " ---"
                for k in sorted(lbvserver.options.iterkeys(), key=lambda k: k):
                        print "\t%s: %s" % (k, lbvserver.options[k])

        if args.object == "server":
                server = NSServer()
                server.set_name(args.object_name)
                server = NSServer.get(nitro, server)
                print "--- Server: " + server.get_name() + " ---"
                for k in sorted(server.options.iterkeys(), key=lambda k: k):
                        print "\t%s: %s" % (k, server.options[k])

        if args.object == "service":
                service = NSService()
                service.set_name(args.object_name)
                service = NSService.get(nitro, service)

                print "--- Service: " + service.get_name() + " ---"
                for k in sorted(service.options.iterkeys(), key=lambda k: k):
                        print "\t%s: %s" % (k, service.options[k])


def action_addservice(args, nitro):
        service = NSService()
        service.set_port(args.port)
        service.set_servicetype(args.servicetype)
        service.set_clttimeout(args.clttimeout)
        service.set_svrtimeout(args.svrtimeout)
        service.set_name(args.service)
        service.set_servername(args.target)
        NSService.add(nitro, service)
        print "Service '%s:%d/%s' was added to '%s'." % (args.service, args.port, args.servicetype, args.target)


def action_addlbvserver(args, nitro):
        lbvserver = NSLBVServer()
        lbvserver.set_name(args.lbvserver)
        lbvserver.set_ipv46(args.ip)
        lbvserver.set_port(args.port)
        lbvserver.set_clttimeout(args.clttimeout)
        lbvserver.set_persistencetype(args.persistencetype)
        lbvserver.set_servicetype(args.servicetype)
        NSLBVServer.add(nitro, lbvserver)
        print "lb vserver %s (%s:%d/%s) was added" % (args.lbvserver, args.ip, args.port, args.servicetype)


def action_enable(args, nitro):
        if args.object == "csvserver":
                csvserver = NSCSVServer()
                csvserver.set_name(args.object_name)
                NSCSVServer.enable(nitro, csvserver)
                print "Enabled cs vserver: %s" % args.object_name

        if args.object == "lbvserver":
                lbvserver = NSLBVServer()
                lbvserver.set_name(args.object_name)
                NSLBVServer.enable(nitro, lbvserver)
                print "Enabled lb vserver: %s" % args.object_name

        if args.object == "server":
                server = NSServer()
                server.set_name(args.object_name)
                NSServer.enable(nitro, server)
                print "Enabled server: %s" % args.object_name

        if args.object == "service":
                service = NSService()
                service.set_name(args.object_name)
                NSService.enable(nitro, service)
                print "Enabled service: %s" % args.object_name


def action_disable(args, nitro):
        if args.object == "csvserver":
                csvserver = NSCSVServer()
                csvserver.set_name(args.object_name)
                NSCSVServer.disable(nitro, csvserver)
                print "Disabled cs vserver: %s" % args.object_name

        if args.object == "lbvserver":
                lbvserver = NSLBVServer()
                lbvserver.set_name(args.object_name)
                NSLBVServer.disable(nitro, lbvserver)
                print "Disabled lb vserver: %s" % args.object_name

        if args.object == "server":
                server = NSServer()
                server.set_name(args.object_name)
                server.set_delay(args.delay)
                server.set_graceful(args.graceful)
                NSServer.disable(nitro, server)
                print "Disabled server: %s" % args.object_name

        if args.object == "service":
                service = NSService()
                service.set_name(args.object_name)
                service.set_delay(args.delay)
                service.set_graceful(args.graceful)
                NSService.disable(nitro, service)
                print "Disabled service: %s" % args.object_name


def action_rename(args, nitro):
        if args.object == "csvserver":
                csvserver = NSCSVServer()
                csvserver.set_name(args.object_name)
                csvserver.set_newname(args.newname)
                NSCSVServer.rename(nitro, csvserver)
                print "Renamed CS vserver from '%s' to '%s'." % (args.object_name, args.newname)

        if args.object == "lbvserver":
                lbvserver = NSLBVServer()
                lbvserver.set_name(args.object_name)
                lbvserver.set_newname(args.newname)
                NSLBVServer.rename(nitro, lbvserver)
                print "Renamed LB vserver from '%s' to '%s'." % (args.object_name, args.newname)

        if args.object == "server":
                server = NSServer()
                server.set_name(args.object_name)
                server.set_newname(args.newname)
                NSServer.rename(nitro, server)
                print "Renamed server from '%s' to '%s'." % (args.object_name, args.newname)

        if args.object == "service":
                service = NSService()
                service.set_name(args.object_name)
                service.set_newname(args.newname)
                NSService.rename(nitro, service)
                print "Renamed service from '%s' to '%s'." % (args.object_name, args.newname)


def action_list(args, nitro):
        if args.object == "csvserver":
                vservers = NSCSVServer().get_all(nitro)
                print "-- Configured CS vservers ---"
                for vserver in sorted(vservers, key=lambda k: k.get_name()):
                        print "\t" + vserver.get_name()

        if args.object == "lbvserver":
                vservers = NSLBVServer().get_all(nitro)
                print "-- Configured LB vservers ---"
                for vserver in sorted(vservers, key=lambda k: k.get_name()):
                        print "\t" + vserver.get_name()

        if args.object == "server":
                servers = NSServer().get_all(nitro)
                print "-- Configured servers ---"
                for server in sorted(servers, key=lambda k: k.get_name()):
                        print "\t" + server.get_name()

        if args.object == "service":
                services = NSService().get_all(nitro)
                print "-- Configured services ---"
                for service in sorted(services, key=lambda k: k.get_name()):
                        print "\t" + service.get_name()


def action_status(args, nitro):
        if args.object == "csvserver":
                vservers = NSCSVServer().get_all(nitro)
                print "-- Configured CS vservers (with status) ---"
                for vserver in sorted(vservers, key=lambda k: k.get_name()):
                        print vserver.get_name() + ": " + vserver.get_curstate()

        if args.object == "lbvserver":
                vservers = NSLBVServer().get_all(nitro)
                print "-- Configured LB vservers (with status) ---"
                for vserver in sorted(vservers, key=lambda k: k.get_name()):
                        print vserver.get_name() + ": " + vserver.get_effectivestate()

        if args.object == "server":
                servers = NSServer().get_all(nitro)
                print "-- Configured servers (with status) ---"
                for server in sorted(servers, key=lambda k: k.get_name()):
                        print "\t" + server.get_name() + ": " + server.get_state()

        if args.object == "service":
                services = NSService().get_all(nitro)
                print "-- Configured services (with status) ---"
                for service in sorted(services, key=lambda k: k.get_name()):
                        print "\t" + service.get_name() + ": " + service.get_svrstate()


def action_statusfull(args, nitro):
        binding = NSLBVServerServiceBinding()
        binding.set_name(args.object_name)
        binded_services = NSLBVServerServiceBinding().get(nitro, binding)
        print "-- | VServerName | Servicename | State | Type | IP | Port | Weight | ---"
        for s in binded_services:
                print ' | ', s.get_name(), ' | ', s.get_servicename(), ' | ', s.get_curstate(),
                print ' | ', s.get_servicetype(), ' | ', s.get_ipv46(), ' | ', s.get_port(),
                print ' | ', s.get_weight()


def action_saveconfig(args, nitro):
        NSConfig.save(nitro)
        print "Saved Netscaler configuration"


def action_bindserver(args, nitro):
        svcgrpbinding = NSServiceGroupServerBinding()
        svcgrpbinding.set_servername(args.server)
        svcgrpbinding.set_servicegroupname(args.servicegroup)
        svcgrpbinding.set_port(args.serviceport)
        try:
                NSServiceGroupServerBinding.add(nitro, svcgrpbinding)
                print "bound server %s to service group %s on port %s" % (
                        args.servername, args.servicegroupname, args.serviceport)
        except nsnitro.nsexceptions.nsexceptions.NSNitroNserrExist as e:
                print "Error: ", e.message


def action_unbindserver(args, nitro):
        svcgrpbinding = NSServiceGroupServerBinding()
        svcgrpbinding.set_servername(args.server)
        svcgrpbinding.set_servicegroupname(args.servicegroup)
        svcgrpbinding.set_port(args.serviceport)
        try:
                NSServiceGroupServerBinding.delete(nitro, svcgrpbinding)
                print "unbound server %s from service group %s on port %s" % (
                        args.servername, args.servicegroupname, args.serviceport)
        except nsnitro.nsexceptions.nsexceptions.NSNitroNserrNoent as e:
                print "Error: ", e.message


def action_bindservice(args, nitro):
        binding = NSLBVServerServiceBinding()
        binding.set_weight(args.bindingweight)
        binding.set_servicename(args.service)
        binding.set_name(args.lbvserver)
        NSLBVServerServiceBinding.add(nitro, binding)
        print "Service '%s' was binded to LB vserver '%s' with weight %d." % (
                args.service, args.lbvserver, args.bindingweight)


def action_unbindservice(args, nitro):
        raise NotImplementedError


def action_acl(args, nitro):
        if args.action == "get":
                acl = NSAcl()
                acl.set_aclname(args.name)
                acl = NSAcl.get(nitro, acl)

                print "--- ACL: " + acl.get_aclname() + " ---"
                for k in sorted(acl.options.iterkeys(), key=lambda k: k):
                        print "\t%s: %s" % (k, acl.options[k])


        if args.action == "list":
                acls = NSAcl().get_all(nitro)
                print "-- Configured ACLs ---"
                for acl in sorted(acls, key=lambda k: k.get_aclname()):
                        print "\t" + acl.get_aclname()

        if args.action == "apply":
                NSAcls.apply(nitro)
                print "Applied all ACLs to netscaler kernel."

        if args.action == "clear":
                NSAcls.clear(nitro)
                print "Cleared ACLs on the netscaler."

        if args.action == "renumber":
                NSAcls.renumber(nitro)
                print "Renumbered ACLs on the netscaler."

        if args.action == "delete":
                if not args.name:
                        print("--name required for deletion")
                else:
                        acl = NSAcl()
                        acl.set_aclname(args.delacl)
                        acl = NSAcl.delete(nitro, acl)
                        print "ACL '%s' was deleted. Do not forget to run --applyacls to activate it." % (args.name)


if __name__ == "__main__":
        OBJECTS = ("csvserver", "lbvserver", "server", "service")

        parser = argparse.ArgumentParser(description='Netscaler NITRO controller')
        # Top-level optional arguments
        parser.add_argument('lbip', metavar='IP', help='lb ip address')
        parser.add_argument('--user', metavar='USERNAME', help='lb username')
        parser.add_argument('--password', metavar='PASSWORD', help='lb password')
        parser.add_argument('--ssl', action="store_true", help='turns SSL on')

        # Parent Parsers
        pp_objects = argparse.ArgumentParser(add_help=False)
        pp_objects.add_argument('object', choices=OBJECTS, help="Object type to operate on")
        pp_objects.add_argument('object_name', metavar="OBJECTNAME", help="Name of object")

        pp_list = argparse.ArgumentParser(add_help=False)
        pp_list.add_argument('object', choices=OBJECTS, help="Object type to list")

        pp_addservice = argparse.ArgumentParser(description="Add a service to a lbvserver", add_help=False)
        pp_addservice.add_argument('service', metavar='NAME', help='name of service to add')
        pp_addservice.add_argument('target', metavar='LBVSERVER', help='target lbvserver to add it to')
        pp_addservice.add_argument('--persistence', metavar='TYPE', default='NONE',
                                   help='persistence type (none by default)')
        pp_addservice.add_argument('ip', metavar='IP', help='IP address')
        pp_addservice.add_argument('port', metavar='PORT', help='port number')
        pp_addservice.add_argument('--clttimeout', metavar='CLTTIMEOUT', default=9000,
                                   help='Clt timeout (9000 by default')
        pp_addservice.add_argument('--svrtimeout', metavar='SRVTIMEOUT', default=9000,
                                   help='service timeout (9000 by default')
        pp_addservice.add_argument('--servicetype', metavar='TYPE', choices=['TCP', 'HTTP'], default='TCP',
                                   help='Service type, TCP by default')

        pp_addlbvserver = argparse.ArgumentParser(description="Add a service to a lbvserver", add_help=False)
        pp_addlbvserver.add_argument('lbvserver', metavar='NAME', help='name of lbvserver to add')
        pp_addlbvserver.add_argument('ip', metavar='IP', help='IP address')
        pp_addlbvserver.add_argument('port', metavar='PORT', help='port number')
        pp_addlbvserver.add_argument('--persistencetype', metavar='PERSISTENCETYPE', default='NONE',
                                     help='persistence type')
        pp_addlbvserver.add_argument('--clttimeout', metavar='CLTTIMEOUT', default=9000, help='Clt timeout')
        pp_addlbvserver.add_argument('--servicetype', metavar='SERVICETYPE', choices=['TCP', 'HTTP'], default='TCP',
                                     help='Service type')

        pp_bindservice = argparse.ArgumentParser(description="Bind a service to a lbvserver", add_help=False)
        pp_bindservice.add_argument('service', metavar='SERVICENAME', help='service to bind')
        pp_bindservice.add_argument('lbvserver', metavar='LBVSERVERNAME', help='lbvserver to bind to')
        pp_bindservice.add_argument('--bindingweight', metavar='BINDINGWIIGHT', default=40,
                                    help='weight parameter for binding service')

        pp_bindserver = argparse.ArgumentParser(description="Bind a server to a service group", add_help=False)
        pp_bindserver.add_argument('server', metavar='SERVERNAME', help='server to bind')
        pp_bindserver.add_argument('servicegroup', metavar='SERVICEGROUPNAME',
                                   help='service group object to bind to')
        pp_bindserver.add_argument('--serviceport', required=True, help='service port')

        pp_disable = argparse.ArgumentParser(description="Disable parameters", add_help=False)
        pp_disable.add_argument('--delay', metavar='DELAY', help='Delay in seconds', default=None)
        pp_disable.add_argument('--graceful', metavar='GRACEFUL', choices=['YES', 'NO'], default='NO',
                                help='Graceful YES | NO')

        pp_rename = argparse.ArgumentParser(description="Rename an object", add_help=False)
        pp_rename.add_argument('newname', metavar='NEWNAME', help="New name for object")

        pp_acl = argparse.ArgumentParser(description="ACL operations", add_help=False)
        pp_acl.add_argument('action', choices=['get', 'list', 'apply', 'clear', 'renumber', 'delete'],
                            help="Action to perform")
        pp_acl.add_argument('--name', metavar="ACLNAME", help="Name of acl when getting/deleting", default=None)

        pp_statusfull = argparse.ArgumentParser(description="Print more detailed lbvserver status", add_help=False)
        pp_statusfull.add_argument('object_name', metavar="LBVSERVERNAME", help="Name of lbvserver")


        # Subparsers
        subparsers = parser.add_subparsers(dest="context")
        sp_get = subparsers.add_parser('get', parents=[pp_objects], help="Get information about an object")
        sp_addlbvserver = subparsers.add_parser('addlbvserver', parents=[pp_addlbvserver], help="Add an lbvserver")
        sp_addservice = subparsers.add_parser('addservice', parents=[pp_addservice], help="Add a service")
        sp_enable = subparsers.add_parser('enable', parents=[pp_objects], help="Enable an object")
        sp_disable = subparsers.add_parser('disable', parents=[pp_objects, pp_disable], help="Disable an object")
        sp_rename = subparsers.add_parser('rename', parents=[pp_objects, pp_rename], help="Rename an object")
        sp_list = subparsers.add_parser('list', parents=[pp_list], help="Get a list of objects")
        sp_status = subparsers.add_parser('status', parents=[pp_list], help="Get status of an object")
        sp_saveconfig = subparsers.add_parser('saveconfig', help="save load balancer config")
        sp_getprimarylb = subparsers.add_parser('getprimarylb', parents=[])
        sp_bindservice = subparsers.add_parser('bindservice', parents=[pp_bindservice],
                                               help="Bind a service to a lbvserver")
        sp_unbindservice = subparsers.add_parser('unbindservice', parents=[pp_bindservice],
                                                 help="Unbind a service from a lbvserver")
        sp_bindserver = subparsers.add_parser('bindserver', parents=[pp_bindserver],
                                              help="Bind a server to a servicegroup")
        sp_unbindserver = subparsers.add_parser('unbindserver', parents=[pp_bindserver],
                                                help="Unbind a server from a servicegroup")
        sp_statusfull = subparsers.add_parser('statusfull', parents=[pp_statusfull], help="Get full status")
        sp_acl = subparsers.add_parser('acl', parents=[pp_acl], help="ACL operations")


        # Set functions
        sp_get.set_defaults(func=action_get)
        sp_addlbvserver.set_defaults(func=action_addlbvserver)
        sp_addservice.set_defaults(func=action_addservice)
        sp_enable.set_defaults(func=action_enable)
        sp_disable.set_defaults(func=action_disable)
        sp_rename.set_defaults(func=action_rename)
        sp_list.set_defaults(func=action_list)
        sp_status.set_defaults(func=action_status)
        sp_saveconfig.set_defaults(func=action_saveconfig)
        sp_getprimarylb.set_defaults(func=action_getprimarylb)
        sp_bindservice.set_defaults(func=action_bindservice)
        sp_unbindservice.set_defaults(func=action_unbindservice)
        sp_bindserver.set_defaults(func=action_bindserver)
        sp_unbindserver.set_defaults(func=action_unbindserver)
        sp_statusfull.set_defaults(func=action_statusfull)
        sp_acl.set_defaults(func=action_acl)
        args = parser.parse_args()

        if not args.user:
                user = getpass.getuser()
        else:
                user = args.user

        if not args.password:
                password = getpass.getpass("{0}@{1}'s password: ".format(user, args.lbip))
        else:
                password = args.password

        nitro = NSNitro(args.lbip, user, password, args.ssl)

        try:
                nitro.login()
        except NSNitroError, e:
                print("Error: %s" % e.message)
                sys.exit(1)

        try:
                args.func(args, nitro)
        finally:
                nitro.logout()
                sys.exit(0)



from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSSSLAction(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSSSLAction, self).__init__()
        self.options = {'name': '',
                        'clientauth': '',
                        'clientcert': '',
                        'certheader': '',
                        'clientcertserialnumber': '',
                        'certserialheader': '',
                        'clientcertsubject': '',
                        'certsubjectheader': '',
                        'clientcerthash': '',
                        'certhashheader': '',
                        'clientcertissuer': '',
                        'certissuerheader': '',
                        'sessionid': '',
                        'sessionidheader': '',
                        'cipher': '',
                        'cipherheader': '',
                        'clientcertnotbefore': '',
                        'certnotbeforeheader': '',
                        'clientcertnotafter': '',
                        'certnotafterheader': '',
                        'owasupport': ''}

        self.resourcetype = NSSSLAction.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        """
        Binding object showing the lbmonitor that can be bound to service.
        """
        return "sslaction"

    # Read/write properties
    def set_name(self, name):
        """
        The name for the new SSL action. Maximum Length: 32.
        Minimum length = 1
        """
        self.options['name'] = name

    def get_name(self):
        """
        The name for the new SSL action. Maximum Length: 32.
        Minimum length = 1
        """
        return self.options['name']

    def set_clientauth(self, clientauth):
        """
        Set action to do client certificate authentication or no
        authentication. DOCLIENTAUTH: Perform client certificate
        authentication. NOCLIENTAUTH: No client certificate authentication.
        """
        self.options['clientauth'] = clientauth

    def get_clientauth(self):
        """
        Set action to do client certificate authentication or no
        authentication. DOCLIENTAUTH: Perform client certificate
        authentication. NOCLIENTAUTH: No client certificate authentication.
        """
        return self.options['clientauth']

    def set_clientcert(self, clientcert):
        """
        The state of insertion of the client certificate in the HTTP header
        when the request is sent to the web-server.
        """
        self.options['clientcert'] = clientcert

    def get_clientcert(self):
        """
        The state of insertion of the client certificate in the HTTP header
        when the request is sent to the web-server.
        """
        return self.options['clientcert']

    def set_certheader(self, certheader):
        """
        The tag name to be used while inserting the client certificate in the
        HTTP header.
        """
        self.options['certheader'] = certheader

    def get_certheader(self):
        """
        The tag name to be used while inserting the client certificate in the
        HTTP header.
        """
        return self.options['certheader']

    def set_clientcertserialnumber(self, clientcertserialnumber):
        """
        The state of insertion of the client certificate's Serial Number in the
        HTTP header when the request is sent to the web-server.
        """
        self.options['clientcertserialnumber'] = clientcertserialnumber

    def get_clientcertserialnumber(self):
        """
        The state of insertion of the client certificate's Serial Number in the
        HTTP header when the request is sent to the web-server.
        """
        return self.options['clientcertserialnumber']

    def set_certserialheader(self, certserialheader):
        """
        The tag name to be used while inserting the client certificate's Serial
        Number in the HTTP header.
        """
        self.options['certserialheader'] = certserialheader

    def get_certserialheader(self):
        """
        The tag name to be used while inserting the client certificate's Serial
        Number in the HTTP header.
        """
        return self.options['certserialheader']

    def set_clientcertsubject(self, clientcertsubject):
        """
        The state of insertion of the client certificate's Subject Name in the
        HTTP header when the request is sent to the web-server.
        """
        self.options['clientcertsubject'] = clientcertsubject

    def get_clientcertsubject(self):
        """
        The state of insertion of the client certificate's Subject Name in the
        HTTP header when the request is sent to the web-server.
        """
        return self.options['clientcertsubject']

    def set_certsubjectheader(self, certsubjectheader):
        """
        The tag name to be used while inserting the client certificate's
        Subject Name in the HTTP header.
        """
        self.options['certsubjectheader'] = certsubjectheader

    def get_certsubjectheader(self):
        """
        The tag name to be used while inserting the client certificate's
        Subject Name in the HTTP header.
        """
        return self.options['certsubjectheader']

    def set_clientcerthash(self, clientcerthash):
        """
        The state of insertion of the client certificate's hash (signature) in
        the HTTP header when the request is sent to the web-server.
        """
        self.options['clientcerthash'] = clientcerthash

    def get_clientcerthash(self):
        """
        The state of insertion of the client certificate's hash (signature) in
        the HTTP header when the request is sent to the web-server.
        """
        return self.options['clientcerthash']

    def set_certhashheader(self, certhashheader):
        """
        The tag name to be used while inserting the client certificate's hash
        (signature) in the HTTP header.
        """
        self.options['certhashheader'] = certhashheader

    def get_certhashheader(self):
        """
        The tag name to be used while inserting the client certificate's hash
        (signature) in the HTTP header.
        """
        return self.options['certhashheader']

    def set_clientcertissuer(self, clientcertissuer):
        """
        The state of insertion of the client certificate's Issuer Name in the
        HTTP header when the request is sent to the web-server.
        """
        self.options['clientcertissuer'] = clientcertissuer

    def get_clientcertissuer(self):
        """
        The state of insertion of the client certificate's Issuer Name in the
        HTTP header when the request is sent to the web-server.
        """
        return self.options['clientcertissuer']

    def set_certissuerheader(self, certissuerheader):
        """
        The tag name to be used while inserting the certificate's Issuer Name
        in the HTTP header.
        """
        self.options['certissuerheader'] = certissuerheader

    def get_certissuerheader(self):
        """
        The tag name to be used while inserting the certificate's Issuer Name
        in the HTTP header.
        """
        return self.options['certissuerheader']

    def set_sessionid(self, sessionid):
        """
        The state of insertion of the Session-ID in the HTTP header when the
        request is sent to the web-server.
        """
        self.options['sessionid'] = sessionid

    def get_sessionid(self):
        """
        The state of insertion of the Session-ID in the HTTP header when the
        request is sent to the web-server.
        """
        return self.options['sessionid']

    def set_sessionidheader(self, sessionidheader):
        """
        The tag name to be used while inserting the Session-ID in the HTTP
        header.
        """
        self.options['sessionidheader'] = sessionidheader

    def get_sessionidheader(self):
        """
        The tag name to be used while inserting the Session-ID in the HTTP
        header.
        """
        return self.options['sessionidheader']

    def set_cipher(self, cipher):
        """
        The state of insertion of the cipher details in the HTTP header when
        the request is sent to the web-server.
        """
        self.options['cipher'] = cipher

    def get_cipher(self):
        """
        The state of insertion of the cipher details in the HTTP header when
        the request is sent to the web-server.
        """
        return self.options['cipher']

    def set_cipherheader(self, cipherheader):
        """
        The tag name to be used while inserting the cipher details in the HTTP
        header.
        """
        self.options['cipherheader'] = cipherheader

    def get_cipherheader(self):
        """
        The tag name to be used while inserting the cipher details in the HTTP
        header.
        """
        return self.options['cipherheader']

    def set_clientcertnotbefore(self, clientcertnotbefore):
        """
        The state of insertion of the client certificate's Not-Before date in
        the HTTP header when the request is sent to the web-server.
        """
        self.options['clientcertnotbefore'] = clientcertnotbefore

    def get_clientcertnotbefore(self):
        """
        The state of insertion of the client certificate's Not-Before date in
        the HTTP header when the request is sent to the web-server.
        """
        return self.options['clientcertnotbefore']

    def set_certnotbeforeheader(self, certnotbeforeheader):
        """
        The tag name to be used while inserting the client certificate's
        Not-Before date in the HTTP header.
        """
        self.options['certnotbeforeheader'] = certnotbeforeheader

    def get_certnotbeforeheader(self):
        """
        The tag name to be used while inserting the client certificate's
        Not-Before date in the HTTP header.
        """
        return self.options['certnotbeforeheader']

    def set_clientcertnotafter(self, clientcertnotafter):
        """
        The state of insertion of the client certificate's Not-After in the
        HTTP header when the request is sent to the web-server.
        """
        self.options['clientcertnotafter'] = clientcertnotafter

    def get_clientcertnotafter(self):
        """
        The state of insertion of the client certificate's Not-After in the
        HTTP header when the request is sent to the web-server.
        """
        return self.options['clientcertnotafter']

    def set_certnotafterheader(self, certnotafterheader):
        """
        The tag name to be used while inserting the client certificate's
        Not-After date in the HTTP header.
        """
        self.options['certnotafterheader'] = certnotafterheader

    def get_certnotafterheader(self):
        """
        The tag name to be used while inserting the client certificate's
        Not-After date in the HTTP header.
        """
        return self.options['certnotafterheader']

    def set_owasupport(self, owasupport):
        """
        The state of Outlook Web-Access support. If the system is in front of
        an Outlook Web Access (OWA) server, a special header field,
        'FRONT-END-HTTPS: ON', needs to be inserted in the HTTP requests going
        to the OWA server. Note: This parameter is required as the SSL requests
        (HTTPS) arrives at the back-end Exchange-2000 server on the configured
        HTTP port (80) instead of arriving at the front-end Exchange 2000
        server.
        """
        self.options['owasupport'] = owasupport

    def get_owasupport(self):
        """
        The state of Outlook Web-Access support. If the system is in front of
        an Outlook Web Access (OWA) server, a special header field,
        'FRONT-END-HTTPS: ON', needs to be inserted in the HTTP requests going
        to the OWA server. Note: This parameter is required as the SSL requests
        (HTTPS) arrives at the back-end Exchange-2000 server on the configured
        HTTP port (80) instead of arriving at the front-end Exchange 2000
        server.
        """
        return self.options['owasupport']

    # Read only properties

    @staticmethod
    def get(nitro, sslaction):
        """
        Use this API to fetch sslaction resource of given name.
        """
        __sslaction = NSSSLAction()
        __sslaction.get_resource(nitro, sslaction.get_name())
        return __sslaction

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured sslpolicy resources.
        """
        __url = nitro.get_url() + NSSSLAction.get_resourcetype()
        __json_sslactions = nitro.get(__url).get_response_field(NSSSLAction.get_resourcetype())
        __sslactions = []
        for json_sslaction in __json_sslactions:
            __sslactions.append(NSSSLAction(json_sslaction))
        return __sslactions

    @staticmethod
    def add(nitro, resource):
        """
        Use this API to add resource.
        """
        __resource = NSSSLAction()
        __resource.set_name(resource.get_name())
        __resource.set_clientauth(resource.get_clientauth())
        __resource.set_clientcert(resource.get_clientcert())
        __resource.set_certheader(resource.get_certheader())
        __resource.set_clientcertserialnumber(resource.get_clientcertserialnumber())
        __resource.set_certserialheader(resource.get_certserialheader())
        __resource.set_clientcertsubject(resource.get_clientcertsubject())
        __resource.set_certsubjectheader(resource.get_certsubjectheader())
        __resource.set_clientcerthash(resource.get_clientcerthash())
        __resource.set_certhashheader(resource.get_certhashheader())
        __resource.set_clientcertissuer(resource.get_clientcertissuer())
        __resource.set_certissuerheader(resource.get_certissuerheader())
        __resource.set_sessionid(resource.get_sessionid())
        __resource.set_sessionidheader(resource.get_sessionidheader())
        __resource.set_cipher(resource.get_cipher())
        __resource.set_cipherheader(resource.get_cipherheader())
        __resource.set_clientcertnotbefore(resource.get_clientcertnotbefore())
        __resource.set_certnotbeforeheader(resource.get_certnotbeforeheader())
        __resource.set_clientcertnotafter(resource.get_clientcertnotafter())
        __resource.set_certnotafterheader(resource.get_certnotafterheader())
        __resource.set_owasupport(resource.get_owasupport())
        return __resource.add_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        """
        Use this API to delete server of a given name.
        """
        __resource = NSSSLAction()
        __resource.set_name(resource.get_name())
        return __resource.delete_resource(nitro)


from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSSSLService(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSSSLService, self).__init__()
        self.options = {'servicename': '',
                        'dh': '',
                        'dhfile': '',
                        'dhcount': '',
                        'ersa': '',
                        'ersacount': '',
                        'sessreuse': '',
                        'sesstimeout': '',
                        'cipherredirect': '',
                        'cipherurl': '',
                        'sslv2redirect': '',
                        'sslv2url': '',
                        'clientauth': '',
                        'clientcert': '',
                        'sslredirect': '',
                        'redirectportrewrite': '',
                        'nonfipsciphers': '',
                        'ssl2': '',
                        'ssl3': '',
                        'tls1': '',
                        'snienable': '',
                        'serverauth': '',
                        'pushenctrigger': '',
                        'cipherdetails': '',
                        'crlcheck': '',
                        'certkeyname': '',
                        'cleartextport': '',
                        'ocspcheck': ''}

        self.resourcetype = NSSSLService.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        """
        Binding object showing the lbmonitor that can be bound to service.
        """
        return "sslservice"

    # Read/write properties
    def set_servicename(self, servicename):
        """
        The SSL service name for which the advance configurations are to be
        set.
        Minimum length = 1
        """
        self.options['servicename'] = servicename

    def get_servicename(self):
        """
        The SSL service name for which the advance configurations are to be
        set.
        Minimum length = 1
        """
        return self.options['servicename']

    def set_dh(self, dh):
        """
        The state of Diffie-Hellman (DH) key exchange support for the SSL
        service.
        Default value: DISABLED
        """
        self.options['dh'] = dh

    def get_dh(self):
        """
        The state of Diffie-Hellman (DH) key exchange support for the SSL
        service.
        Default value: DISABLED
        """
        return self.options['dh']

    def set_dhfile(self, dhfile):
        """
        The file name and path for the DH parameter. You need to enable the -dh
        option. File format is PEM. The default input path for the DH file is
        /nsconfig/ssl/.
        Minimum length = 1
        """
        self.options['dhfile'] = dhfile

    def get_dhfile(self):
        """
        The file name and path for the DH parameter. You need to enable the -dh
        option. File format is PEM. The default input path for the DH file is
        /nsconfig/ssl/.
        Minimum length = 1
        """
        return self.options['dhfile']

    def set_dhcount(self, dhcount):
        """
        The refresh count for regeneration of DH public-key and private-key
        from the DH parameter. Zero means infinite usage (no refresh). Option
        '-dh' has to be enabled.
        Default value: 0
        Minimum value = 0
        Maximum value = 65534
        """
        self.options['dhcount'] = dhcount

    def get_dhcount(self):
        """
        The refresh count for regeneration of DH public-key and private-key
        from the DH parameter. Zero means infinite usage (no refresh). Option
        '-dh' has to be enabled.
        Default value: 0
        Minimum value = 0
        Maximum value = 65534
        """
        return self.options['dhcount']

    def set_ersa(self, ersa):
        """
        The state of Ephemeral RSA key exchange support for the SSL service.
        Default value: ENABLED
        """
        self.options['ersa'] = ersa

    def get_ersa(self, ersa):
        """
        The state of Ephemeral RSA key exchange support for the SSL service.
        Default value: ENABLED
        """
        return self.options['ersa']

    def set_ersacount(self, ersacount):
        """
        The refresh count for re-generation of RSA public-key and private-key
        pair. Zero means infinite usage (no refresh). Option '-eRSA' has to be
        enabled.
        Default value: 0
        Minimum value = 0
        Maximum value = 65534
        """
        self.options['ersacount'] = ersacount

    def get_ersacount(self):
        """
        The refresh count for re-generation of RSA public-key and private-key
        pair. Zero means infinite usage (no refresh). Option '-eRSA' has to be
        enabled.
        Default value: 0
        Minimum value = 0
        Maximum value = 65534
        """
        return self.options['ersacount']

    def set_sessreuse(self, sessreuse):
        """
        The state of session reuse support for the SSL service.
        Default value: ENABLED
        """
        self.options['sessreuse'] = sessreuse

    def get_sessreuse(self):
        """
        The state of session reuse support for the SSL service.
        Default value: ENABLED
        """
        return self.options['sessreuse']

    def set_sesstimeout(self, sesstimeout):
        """
        The session timeout value in seconds. The value has to be a positive
        integer. Option '-sessReuse' has to be enabled.
        Default value: 300
        Minimum value = 0
        Maximum value = 0xFFFFFFFE
        """
        self.options['sesstimeout'] = sesstimeout

    def get_sesstimeout(self):
        """
        The session timeout value in seconds. The value has to be a positive
        integer. Option '-sessReuse' has to be enabled.
        Default value: 300
        Minimum value = 0
        Maximum value = 0xFFFFFFFE
        """
        return self.options['sesstimeout']

    def set_cipherredirect(self, cipherredirect):
        """
        The state of Cipher Redirect feature.
        Default value: DISABLED
        """
        self.options['cipherredirect'] = cipherredirect

    def get_cipherredirect(self):
        """
        The state of Cipher Redirect feature.
        Default value: DISABLED
        """
        return self.options['cipherredirect']

    def set_cipherurl(self, cipherurl):
        """
        The redirect URL to be used with the Cipher Redirect feature.
        """
        self.options['cipherurl'] = cipherurl

    def get_cipherurl(self):
        """
        The redirect URL to be used with the Cipher Redirect feature.
        """
        return self.options['cipherurl']

    def set_sslv2redirect(self, sslv2redirect):
        """
        The state of SSLv2 Redirect feature.
        Default value: DISABLED
        """
        self.options['sslv2redirect'] = sslv2redirect

    def get_sslv2redirect(self):
        """
        The state of SSLv2 Redirect feature.
        Default value: DISABLED
        """
        return self.options['sslv2redirect']

    def set_sslv2url(self, sslv2url):
        """
        The redirect URL to be used with the SSLv2 Redirect feature.
        """
        self.options['sslv2url'] = sslv2url

    def get_sslv2url(self):
        """
        The redirect URL to be used with the SSLv2 Redirect feature.
        """
        return self.options['sslv2url']

    def set_clientauth(self, clientauth):
        """
        The state of Client-Authentication support for the SSL service.
        Default value: DISABLED
        """
        self.options['clientauth'] = clientauth

    def get_clientauth(self):
        """
        The state of Client-Authentication support for the SSL service.
        Default value: DISABLED
        """
        return self.options['clientauth']

    def set_clientcert(self, clientcert):
        """
        The rule for client authentication. If clientCert is set to Mandatory,
        System will terminate the SSL handshake if SSL client does not provide
        a valid certificate. If the setting is optional, then System will allow
        SSL clients with no certificate or invalid certificates to access the
        secure resource. Note: Make sure proper access control policies are
        defined before changing the above setting to Optional.
        """
        self.options['clientcert'] = clientcert

    def get_clientcert(self):
        """
        The rule for client authentication. If clientCert is set to Mandatory,
        System will terminate the SSL handshake if SSL client does not provide
        a valid certificate. If the setting is optional, then System will allow
        SSL clients with no certificate or invalid certificates to access the
        secure resource. Note: Make sure proper access control policies are
        defined before changing the above setting to Optional.
        """
        return self.options['clientcert']

    def set_sslredirect(self, sslredirect):
        """
        The state of HTTPS redirects for the SSL service. This is required for
        the proper functioning of the redirect messages from the server. The
        redirect message from the server provides the new location for the
        moved object. This is contained in the HTTP header field: Location,
        e.g. Location: http://www.moved.org/here.html For the SSL session, if
        the client browser receives this message, the browser will try to
        connect to the new location. This will break the secure SSL session, as
        the object has moved from a secure site (https://) to an un-secure one
        (http://). Generally browsers flash a warning message on the screen and
        prompt the user, either to continue or disconnect. The above feature,
        when enabled will automatically convert all such http:// redirect
        message to https://. This will not break the client SSL session. Note:
        The set ssl service command can be used for configuring a front-end SSL
        service for service based SSL Off-Loading, or a backend SSL service for
        backend-encryption setup. Some of the command options are not
        applicable while configuring a backend service. System will not report
        an error if these options are used for a backend SSL service. These
        are: [-dh (ENABLED|DISABLED) (-dhFile < file_name >)] [(-dhCount )]
        [-eRSA (ENABLED|DISABLED)] [(-eRSACount )] [-cipherRedirect (ENABLED |
        DISABLED) [-cipherURL ]] [-sslv2Redirect ( ENABLED | DISABLED )
        [-sslv2URL ]] [-clientAuth ( ENABLED | DISABLED ) [-clientCert (
        Mandatory | Optional )]] [-sslRedirect ( ENABLED | DISABLED )] [-ssl2
        (ENABLED|DISABLED)].
        Default value: DISABLED
        """
        self.options['sslredirect'] = sslredirect

    def get_sslredirect(self):
        """
        The state of HTTPS redirects for the SSL service. This is required for
        the proper functioning of the redirect messages from the server. The
        redirect message from the server provides the new location for the
        moved object. This is contained in the HTTP header field: Location,
        e.g. Location: http://www.moved.org/here.html For the SSL session, if
        the client browser receives this message, the browser will try to
        connect to the new location. This will break the secure SSL session, as
        the object has moved from a secure site (https://) to an un-secure one
        (http://). Generally browsers flash a warning message on the screen and
        prompt the user, either to continue or disconnect. The above feature,
        when enabled will automatically convert all such http:// redirect
        message to https://. This will not break the client SSL session. Note:
        The set ssl service command can be used for configuring a front-end SSL
        service for service based SSL Off-Loading, or a backend SSL service for
        backend-encryption setup. Some of the command options are not
        applicable while configuring a backend service. System will not report
        an error if these options are used for a backend SSL service. These
        are: [-dh (ENABLED|DISABLED) (-dhFile < file_name >)] [(-dhCount )]
        [-eRSA (ENABLED|DISABLED)] [(-eRSACount )] [-cipherRedirect (ENABLED |
        DISABLED) [-cipherURL ]] [-sslv2Redirect ( ENABLED | DISABLED )
        [-sslv2URL ]] [-clientAuth ( ENABLED | DISABLED ) [-clientCert (
        Mandatory | Optional )]] [-sslRedirect ( ENABLED | DISABLED )] [-ssl2
        (ENABLED|DISABLED)].
        Default value: DISABLED
        """
        return self.options['sslredirect']

    def set_redirectportrewrite(self, redirectportrewrite):
        """
        The state of the port in rewrite while performing HTTPS redirect.
        Default value: DISABLED
        """
        self.options['redirectportrewrite'] = redirectportrewrite

    def get_redirectportrewrite(self):
        """
        The state of the port in rewrite while performing HTTPS redirect.
        Default value: DISABLED
        """
        return self.options['redirectportrewrite']

    def set_nonfipsciphers(self, nonfipsciphers):
        """
        The state of usage of non FIPS approved ciphers. Valid only for an SSL
        service bound with a FIPS key and certificate.
        Default value: DISABLED
        """
        self.options['nonfipsciphers'] = nonfipsciphers

    def get_nonfipsciphers(self):
        """
        The state of usage of non FIPS approved ciphers. Valid only for an SSL
        service bound with a FIPS key and certificate.
        Default value: DISABLED
        """
        return self.options['nonfipsciphers']

    def set_ssl2(self, ssl2):
        """
        The state of SSLv2 protocol support for the SSL service.
        Default value: DISABLED
        """
        self.options['ssl2'] = ssl2

    def get_ssl2(self):
        """
        The state of SSLv2 protocol support for the SSL service.
        Default value: DISABLED
        """
        return self.options['ssl2']

    def set_ssl3(self, ssl3):
        """
        The state of SSLv3 protocol support for the SSL service.
        Default value: ENABLED
        """
        self.options['ssl3'] = ssl3

    def get_ssl3(self):
        """
        The state of SSLv3 protocol support for the SSL service.
        Default value: ENABLED
        """
        return self.options['ssl3']

    def set_tls1(self, tls1):
        """
        The state of TLSv1 protocol support for the SSL service.
        Default value: ENABLED
        """
        self.options['tls1'] = tls1

    def get_tls1(self):
        """
        The state of TLSv1 protocol support for the SSL service.
        Default value: ENABLED
        """
        return self.options['tls1']

    def set_snienable(self, snienable):
        """
        state of SNI feature on service.
        Default value: DISABLED
        """
        self.options['snienable'] = snienable

    def get_snienable(self):
        """
        state of SNI feature on service.
        Default value: DISABLED
        """
        return self.options['snienable']


    def set_serverauth(self, serverauth):
        """
        The state of Server-Authentication support for the SSL service.
        Default value: DISABLED
        """
        self.options['serverauth'] = serverauth

    def get_serverauth(self):
        """
        The state of Server-Authentication support for the SSL service.
        Default value: DISABLED
        """
        return self.options['serverauth']

    def set_pushenctrigger(self, pushenctrigger):
        """
        PUSH packet triggering encryption Always - Any PUSH packet triggers
        encryption Ignore - Ignore PUSH packet for triggering encryption Merge
        - For consecutive sequence of PUSH packets, last PUSH packet triggers
        encryption Timer - PUSH packet triggering encryption delayed by timer
        period defined in 'set ssl parameter' .
        """
        self.options['pushenctrigger'] = pushenctrigger

    def get_pushenctrigger(self):
        """
        PUSH packet triggering encryption Always - Any PUSH packet triggers
        encryption Ignore - Ignore PUSH packet for triggering encryption Merge
        - For consecutive sequence of PUSH packets, last PUSH packet triggers
        encryption Timer - PUSH packet triggering encryption delayed by timer
        period defined in 'set ssl parameter' .
        """
        return self.options['pushenctrigger']

    def set_cipherdetails(self, cipherdetails):
        """
        Details of the individual ciphers bound to the SSL service. Select this
        flag value to display the details of the individual ciphers bound to
        the SSL service.
        """
        self.options['cipherdetails'] = cipherdetails

    def get_cipherdetails(self):
        """
        Details of the individual ciphers bound to the SSL service. Select this
        flag value to display the details of the individual ciphers bound to
        the SSL service.
        """
        return self.options['cipherdetails']

    # Read only properties
    def get_crlcheck(self):
        """
        The state of the CRL check parameter. (Mandatory/Optional)
        """
        return self.options['crlcheck']

    def get_certkeyname(self):
        """
        The certificate key pair binding.
        """
        return self.options['certkeyname']

    def get_cleartextport(self):
        """
        The clearTextPort settings.
        """
        return self.options['cleartextport']

    def get_ocspcheck(self):
        """
        The state of the OCSP check parameter. (Mandatory/Optional)
        """
        return self.options['ocspcheck']

    @staticmethod
    def get(nitro, sslservice):
        """
        Use this API to fetch sslservice resource of given name.
        """
        __sslservice = NSSSLService()
        __sslservice.get_resource(nitro, sslservice.get_servicename())
        return __sslservice

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured sslservice resources.
        """
        __url = nitro.get_url() + NSSSLService.get_resourcetype()
        __json_sslservices = nitro.get(__url).get_response_field(NSSSLService.get_resourcetype())
        __sslservices = []
        for json_sslservice in __json_sslservices:
            __sslservices.append(NSSSLService(json_sslservice))
        return __sslservices

    @staticmethod
    def update(nitro, sslservice):
        """
        Use this API to update sslservice of a given name.
        """
        __sslservice = NSSSLService()
        __sslservice.set_servicename(sslservice.get_servicename())
        __sslservice.set_dh(sslservice.get_dh())
        __sslservice.set_dhfile(sslservice.get_dhfile())
        __sslservice.set_dhcount(sslservice.get_dhcount())
        __sslservice.set_ersa(sslservice.get_ersa())
        __sslservice.set_ersacount(sslservice.get_ersacount())
        __sslservice.set_sessreuse(sslservice.get_sessreuse())
        __sslservice.set_sesstimeout(sslservice.get_sesstimeout())
        __sslservice.set_cipherredirect(sslservice.get_cipherredirect())
        __sslservice.set_cipherurl(sslservice.get_cipherurl())
        __sslservice.set_sslv2redirect(sslservice.get_sslv2redirect())
        __sslservice.set_sslv2url(sslservice.get_sslv2redirect())
        __sslservice.set_clientauth(sslservice.get_clientauth())
        __sslservice.set_clientcert(sslservice.get_clientcert())
        __sslservice.set_sslredirect(sslservice.get_sslredirect())
        __sslservice.set_redirectportrewrite(sslservice.get_redirectportrewrite())
        __sslservice.set_nonfipsciphers(sslservice.get_nonfipsciphers())
        __sslservice.set_ssl2(sslservice.get_ssl2())
        __sslservice.set_ssl3(sslservice.get_ssl3())
        __sslservice.set_tls1(sslservice.get_tls1())
        __sslservice.set_snienable(sslservice.get_snienable())
        __sslservice.set_serverauth(sslservice.get_serverauth())
        __sslservice.set_pushenctrigger(sslservice.get_pushenctrigger())
        return __sslvservice.update_resource(nitro)


from nsbaseresource import NSBaseResource

__author__ = 'Aleksandar Topuzovic'


class NSHTTPProfile(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSHTTPProfile, self).__init__()

        self.options = {
            # Read-write values
            'name': '',
            'dropinvalreqs': '',
            'markhttp09inval': '',
            'markconnreqinval': '',
            'cmponpush': '',
            'conmultiplex': '',
            'maxreusepool': '',
            'dropextracrlf': '',
            'incomphdrdelay': '',
            'reqtimeout': '',
            'dropextradata': '',
            'maxreq': '',
            'reusepooltimeout': '',

            # Readonly values
            'refcnt': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

        self.resourcetype = NSHTTPProfile.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "nshttpprofile"

    # Read-write
    def get_name(self):
        """
        Name of the HTTP profile.
        Minimum length = 1
        Maximum length = 127
        """
        return self.options['name']

    def set_name(self, name):
        """
        Name of the HTTP profile.
        Minimum length = 1
        Maximum length = 127
        """
        self.options['name'] = name

    def get_dropinvalreqs(self):
        """
        Enable/disable dropping of invalid HTTP requests/responses.
        Default value: DISABLED
        """
        return self.options['dropinvalreqs']

    def set_dropinvalreqs(self, dropinvalreqs):
        """
        Enable/disable dropping of invalid HTTP requests/responses.
        Default value: DISABLED
        """
        self.options['dropinvalreqs'] = dropinvalreqs

    def get_markhttp09inval(self):
        """
        Enable/disable invalidating HTTP 0.9 requests.
        Default value: DISABLED
        """
        return self.options['markhttp09inval']

    def set_markhttp09inval(self, markhttp09inval):
        """
        Enable/disable invalidating HTTP 0.9 requests.
        Default value: DISABLED
        """
        self.options['markhttp09inval'] = markhttp09inval

    def get_markconnreqinval(self):
        """
        Enable/disable invalidating CONNECT HTTP requests.
        Default value: DISABLED
        """
        return self.options['markconnreqinval']

    def set_markconnreqinval(self, markconnreqinval):
        """
        Enable/disable invalidating CONNECT HTTP requests.
        Default value: DISABLED
        """
        self.options['markconnreqinval'] = markconnreqinval

    def get_cmponpush(self):
        """
        Enable/disable compression on PUSH packet.
        Default value: DISABLED
        """
        return self.options['cmponpush']

    def set_cmponpush(self, cmponpush):
        """
        Enable/disable compression on PUSH packet.
        Default value: DISABLED
        """
        self.options['cmponpush'] = cmponpush

    def get_conmultiplex(self):
        """
        Connection multiplexing.
        Default value: ENABLED
        """
        return self.options['conmultiplex']

    def set_conmultiplex(self, conmultiplex):
        """
        Connection multiplexing.
        Default value: ENABLED
        """
        self.options['conmultiplex'] = conmultiplex

    def get_maxreusepool(self):
        """
        Maximum connections in reusepool. If set to zero, limit will not be applied.
        Default value: 0
        """
        return self.options['maxreusepool']

    def set_maxreusepool(self, maxreusepool):
        """
        Maximum connections in reusepool. If set to zero, limit will not be applied.
        Default value: 0
        """
        self.options['maxreusepool'] = maxreusepool

    def get_dropextracrlf(self):
        """
        Drop extra CRLF after header is complete.
        Default value: ENABLED
        """
        return self.options['dropextracrlf']

    def set_dropextracrlf(self, dropextracrlf):
        """
        Drop extra CRLF after header is complete.
        Default value: ENABLED
        """
        self.options['dropextracrlf'] = dropextracrlf

    def get_incomphdrdelay(self):
        """
        Maximum time to wait between incomplete header packets (ms ticks).
        Default value: 7000
        """
        return self.options['incomphdrdelay']

    def set_incomphdrdelay(self, incomphdrdelay):
        """
        Maximum time to wait between incomplete header packets (ms ticks).
        Default value: 7000
        """
        self.options['incomphdrdelay'] = incomphdrdelay

    def get_reqtimeout(self):
        """
        The time (in seconds) within which the HTTP request must complete.
        Default value: 0
        """
        return self.options['reqtimeout']

    def set_reqtimeout(self, reqtimeout):
        """
        The time (in seconds) within which the HTTP request must complete.
        Default value: 0
        """
        self.options['reqtimeout'] = reqtimeout

    def get_dropextradata(self):
        """
        Drop extra data coming from Server.
        Default value: DISABLED
        """
        return self.options['dropextradata']

    def set_dropextradata(self, dropextradata):
        """
        Drop extra data coming from Server.
        Default value: DISABLED
        """
        self.options['dropextradata'] = dropextradata

    def get_maxreq(self):
        """
        Maximum requests allowed on a single connection.
        Default value: 0
        """
        return self.options['maxreq']

    def set_maxreq(self, maxreq):
        """
        Maximum requests allowed on a single connection.
        Default value: 0
        """
        self.options['maxreq'] = maxreq

    def get_reusepooltimeout(self):
        """
        Idle timeout (in seconds) for server connections in re-use pool.
        Connections in the re-use pool are flushed, if they remain idle for the
        configured timeout.
        Default value: 0
        """
        return self.options['reusepooltimeout']

    def set_reusepooltimeout(self, reusepooltimeout):
        """
        Idle timeout (in seconds) for server connections in re-use pool.
        Connections in the re-use pool are flushed, if they remain idle for the
        configured timeout.
        Default value: 0
        """
        self.options['reusepooltimeout'] = reusepooltimeout

    # Read-only
    def get_refcnt(self):
        """
        Number of entities using this profile
        """
        return self.options['refcnt']

    # Operations methods

    @staticmethod
    def get(nitro, httpprofile):
        """
        Use this API to fetch service resource of given name.
        """
        __httpprofile = NSHTTPProfile()
        __httpprofile.set_name(httpprofile.get_name())
        __httpprofile.get_resource(nitro)
        return __httpprofile

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured service resources.
        """
        __url = nitro.get_url() + NSHTTPProfile.get_resourcetype()
        __json_httpprofiles = nitro.get(__url).get_response_field(NSHTTPProfile.get_resourcetype())
        __httpprofiles = []
        for json_service in __json_httpprofiles:
            __httpprofiles.append(NSHTTPProfile(json_service))
        return __httpprofiles

    @staticmethod
    def add(nitro, httpprofile):
        """
        Use this API to add a http profile.
        """
        __httpprofile = NSHTTPProfile()
        __httpprofile.set_name(httpprofile.get_name())
        __httpprofile.set_dropinvalreqs(httpprofile.get_dropinvalreqs())
        __httpprofile.set_markhttp09inval(httpprofile.get_markhttp09inval())
        __httpprofile.set_markconnreqinval(httpprofile.get_markconnreqinval())
        __httpprofile.set_cmponpush(httpprofile.get_cmponpush())
        __httpprofile.set_conmultiplex(httpprofile.get_conmultiplex())
        __httpprofile.set_maxreusepool(httpprofile.get_maxreusepool())
        __httpprofile.set_dropextracrlf(httpprofile.get_dropextracrlf())
        __httpprofile.set_incomphdrdelay(httpprofile.get_incomphdrdelay())
        __httpprofile.set_reqtimeout(httpprofile.get_reqtimeout())
        __httpprofile.set_dropextradata(httpprofile.get_dropextradata())
        __httpprofile.set_maxreq(httpprofile.get_maxreq())
        __httpprofile.set_reusepooltimeout(httpprofile.get_reusepooltimeout())
        return __httpprofile.add_resource(nitro)

    @staticmethod
    def update(nitro, httpprofile):
        """
        Use this API to update a http profile.
        """
        __httpprofile = NSHTTPProfile()
        __httpprofile.set_name(httpprofile.get_name())
        __httpprofile.set_dropinvalreqs(httpprofile.get_dropinvalreqs())
        __httpprofile.set_markhttp09inval(httpprofile.get_markhttp09inval())
        __httpprofile.set_markconnreqinval(httpprofile.get_markconnreqinval())
        __httpprofile.set_cmponpush(httpprofile.get_cmponpush())
        __httpprofile.set_conmultiplex(httpprofile.get_conmultiplex())
        __httpprofile.set_maxreusepool(httpprofile.get_maxreusepool())
        __httpprofile.set_dropextracrlf(httpprofile.get_dropextracrlf())
        __httpprofile.set_incomphdrdelay(httpprofile.get_incomphdrdelay())
        __httpprofile.set_reqtimeout(httpprofile.get_reqtimeout())
        __httpprofile.set_dropextradata(httpprofile.get_dropextradata())
        __httpprofile.set_maxreq(httpprofile.get_maxreq())
        __httpprofile.set_reusepooltimeout(httpprofile.get_reusepooltimeout())
        return __httpprofile.update_resource(nitro)

    @staticmethod
    def delete(nitro, httpprofile):
        """
        Use this API to delete a httpprofile of a given name.
        """
        __httpprofile = NSHTTPProfile()
        __httpprofile.set_name(httpprofile.get_name())
        nsresponse = __httpprofile.delete_resource(nitro)
        return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSVLANNSIPBinding(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSVLANNSIPBinding, self).__init__()

        self.options = {
            'ipaddress': '',
            'netmask': '',
            'rnat': '',
            'id': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSVLANNSIPBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "vlan_nsip_binding"

    def set_ipaddress(self, ipaddress):
        self.options['ipaddress'] = ipaddress

    def get_ipaddress(self):
        return self.options['ipaddress']

    def set_netmask(self, netmask):
        self.options['netmask'] = netmask

    def get_netmask(self):
        return self.options['netmask']

    def set_rnat(self, rnat):
        self.options['rnat'] = rnat

    def get_rnat(self):
        return self.options['rnat']

    def set_id(self, id):
        self.options['id'] = id

    def get_id(self):
        return self.options['id']

    @staticmethod
    def add(nitro, resource):
        __resource = NSVLANNSIPBinding()
        __resource.set_id(resource.get_id())
        __resource.set_ipaddress(resource.get_ipaddress())
        __resource.set_netmask(resource.get_netmask())
        return __resource.add_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        __resource = NSVLANNSIPBinding()
        __resource.set_id(resource.get_id())
        __resource.set_ipaddress(resource.get_ipaddress())
        __resource.set_netmask(resource.get_netmask())
        nsresponse = __resource.delete_resource(nitro, object_name=__resource.get_id())
        return nsresponse

    @staticmethod
    def get(nitro, resource):
        """
        Use this API to fetch service resource of given name.
        """
        __resource = NSVLANNSIPBinding()
        __resource.set_id(resource.get_id())
        __resource.get_resource(nitro, object_name=resource.get_id())
        return __resource
